require('dotenv').config()
const express = require("express");
const app = express();
const cors = require("cors");
const bodyParser = require("body-parser");
const Agora = require("agora-access-token");
const server = require('http').createServer(app)
const io = require('socket.io')(server)
const schedule = require('node-schedule');
const moment = require('moment');

global.basedir = __dirname;
global.APP_NAME = process.env.APPNAME;
global.APP_URL = process.env.APPURL || "http://localhost:3030";
var corsOptions = {
    origin: ['https://admin.vetzy.ae', global.APP_URL]
};

app.use(cors(corsOptions));

// parse requests of content-type - application/json
app.use(bodyParser.urlencoded({ extended: true }));
app.use(express.json())
app.use('/public', express.static(`${__dirname}/public`));


// simple route
app.get("/", (req, res) => {
    res.json({ message: "Welcome to VEZTY api." });
});

schedule.scheduleJob('*/5 * * * *', () => {
    veterinarianController.cronRefund();
    // console.log("========cron Schedule working=========")
});

require('./models/db');
require("./routes")(app);
const ProductMaster = require('./models/ProductMaster');
// set port, listen for requests
const PORT = process.env.PORT || 3030;
server.listen(PORT, () => {
    console.log(`Server is running on port ${PORT}.`);
});

const { sendFirebaseNotification } = require('./utils/notification')
/* AGORA_TOKEN */
async function getRTCToken(channel, uid = "", isPublisher = "false") {
    //client account
    const appID = "4ce799012a864610b5f8a7277f467e39";
    const appCertificate = "dd4bda431a294d8484aa5b6735c74fe0";
    const expirationTimeInSeconds = 86400;
    const currentTimestamp = Math.floor(Date.now() / 1000);
    const expirationTimestamp = currentTimestamp + expirationTimeInSeconds;
    uid = uid ? uid : Math.floor(Math.random() * 100000);
    role = isPublisher ? Agora.RtcRole.PUBLISHER : Agora.RtcRole.SUBSCRIBER;
    channel = channel ? channel : "fromUserId_toUserId";
    var token = "";

    const tokenPromise = new Promise((resolve, reject) => {
        token = Agora.RtcTokenBuilder.buildTokenWithUid(appID, appCertificate, channel, uid, role, expirationTimestamp);
        resolve();
    });
    await tokenPromise;
    return { token, uid, channel };
}
app.post("/agoraToken", async (req, res) => {
    const uid = req.body.uid;
    const to_user_id = req.body.to_user_id;
    const role = req.body.isPublisher ?? true;

    var fromId = uid;
    var toUserId = to_user_id;
    var manualChannel = fromId + "_" + toUserId;
    if (fromId > toUserId) {
        manualChannel = toUserId + "_" + fromId;
    }
    let channel = req.body.channel ?? manualChannel;
    let randomID = Math.floor(Math.random() * 100000);
    var tokenData = await getRTCToken(channel, randomID, role);
    var fromObject = { uid: uid, channel: channel, token: tokenData.token, randomID: randomID };
    const doctorDetails = await UserMaster.findOne({ _id: uid })
    var agoraToken = {
        token: tokenData.token.toString(),
        channel: tokenData.channel.toString(),
        uid: to_user_id,
        to_user_id: uid,
        doctorName: `${doctorDetails.fname} ${doctorDetails.lname}`,
        doctorImg: `${global.APP_URL}/public/profile/${doctorDetails?.profile}`,
        timestamp: moment().format('YYYY-MM-DD HH:mm:ss')
    }

    // await messageMaster({
    //     sender_id: uid,
    //     receiver_id: to_user_id,
    //     msg: 'Call Started',
    //     seen: false,
    //     msg_type: 'Call Started'
    // }).save()

    console.log("🚀 ~ app.post ~ req.body.channel:", req.body.channel)
    if (!req.body.channel) {          // if user already in call to other user that time not get channel in body
        var findDeviceToken = await UserMaster.findOne({ _id: to_user_id })
        console.log("🚀 ~ app.post ~ findDeviceToken:", findDeviceToken)
        console.log("🚀 ~ app.post ~ findDeviceToken.device_token:", findDeviceToken.device_token)
        if (findDeviceToken && findDeviceToken.device_token != "" && findDeviceToken.device_token != null) {
            await sendFirebaseNotification(findDeviceToken.device_token, 'VETZY', `Incoming Video Call`, agoraToken)
        }
    }

    res.send({ fromObject: fromObject });
});

/* SOCKET IO */
const messageMaster = require('./models/MessageMaster')
const { ObjectId } = require('mongodb')
io.on('connection', (socket) => {
    console.log("connected user", socket.id)

    socket.on('socketJoin', (userId) => {
        if (!socket.rooms.has(userId.toString())) {
            console.log('roomJoin-->>>', userId);
            socket.join(userId.toString());
        }
    });

    socket.on('socketLeave', (userId) => {
        if (socket.rooms.has(userId.toString())) {
            console.log('roomLeave-->>>', userId);
            socket.leave(userId.toString());
        }
    });

    socket.on('openChat', async (login_id, receiver_id) => {
        await UserMaster.findOneAndUpdate({ _id: login_id }, { is_on_chat: receiver_id }, { new: true })
    });

    socket.on('closeChat', async (login_id) => {
        await UserMaster.findOneAndUpdate({ _id: login_id }, { is_on_chat: null }, { new: true })
    });

    socket.on('call_request', (receiver_id, call_req) => {
        socket.to(receiver_id.toString()).emit('call_answer', {
            resData: call_req
        });
    });

    // socket.on('appointmentStatusMsg', async (sender_id, receiver_id, is_status) => {
    //     let msg_type;
    //     if (is_status == "accept") {
    //         msg_type = 'Consultation Accepted'
    //     } else if (is_status == "complete") {
    //         msg_type = 'Consultation Completed'
    //     } else {
    //         msg_type = 'Consultation Rejected'
    //     }
    //     await messageMaster.create({
    //         sender_id: sender_id,
    //         receiver_id: receiver_id,
    //         msg: msg_type,
    //         msg_type: msg_type
    //     });
    //     socket.to(receiver_id.toString()).emit('setAppointmentStatusMsg', {
    //         resData: call_req
    //     });
    // });

    // socket.on('createPricribeMsg', async (sender_id, receiver_id) => {

    //     await messageMaster.create({
    //         sender_id: sender_id,
    //         receiver_id: receiver_id,
    //         msg: 'Prescribe Medicine',
    //         msg_type: 'Prescribe Medicine'
    //     });
    //     socket.to(receiver_id.toString()).emit('setcreatePricribeMsg', {
    //         resData: call_req
    //     });
    // });

    socket.on('sendMessage', async (sender_id, receiver_id, data, msg_type) => {
        try {
            let newMsg = new messageMaster({
                sender_id: sender_id,
                receiver_id: receiver_id,
                msg: data,
                time: moment().format('MMMM Do YYYY, h:mm:ss a'),
                msg_type: msg_type,
            });
            const findUser = await UserMaster.findOne({ _id: receiver_id })
            var result = await newMsg.save();
            const msgData = {
                sender_id: sender_id.toString(),
                receiver_id: receiver_id.toString(),
                msg: data.toString(),
                time: moment().format('MMMM Do YYYY, h:mm:ss a').toString(),
                msg_type: msg_type,
            }
            const senderData = await UserMaster.findOne({ _id: sender_id })
            if (findUser && findUser.device_token && findUser.is_on_chat != sender_id) {
                if (msg_type == "text") sendFirebaseNotification(findUser.device_token, 'VETZY', `${senderData.fname} ${senderData.lname} Sent You a Message`, msgData)
            }
            socket.to(receiver_id).emit('setNewMessage', {
                resData: result
            });
        } catch (error) {
            let errorMsg = error.message ?? "Somethig went wrong";
            console.log('errorMsg-setNewMessage->>', errorMsg);
            socket.emit("setNewMessage", errorMsg);
        }
    });

    // socket.on('getChatUserList', async (login_id) => {
    //     try {
    //         if (!socket.rooms.hasOwnProperty(login_id.toString())) {
    //             socket.join(login_id.toString());
    //         }

    //         // Aggregation to find the latest message for each user
    //         const result = await messageMaster.aggregate([
    //             {
    //                 $match: {
    //                     $or: [
    //                         { sender_id: ObjectId(login_id) },
    //                         { receiver_id: ObjectId(login_id) }
    //                     ],
    //                 }
    //             },
    //             {
    //                 $sort: { created_at: -1 }
    //             },
    //             {
    //                 $group: {
    //                     _id: {
    //                         $cond: [
    //                             { $eq: ["$sender_id", ObjectId(login_id)] },
    //                             "$receiver_id",
    //                             "$sender_id"
    //                         ]
    //                     },
    //                     latestMessage: { $first: "$$ROOT" }
    //                 }
    //             },
    //         ]);

    //         const userDetailPromises = result.map(async (user) => {
    //             const findconsultation = await ConsultationMaster.findOne({
    //                 $or: [
    //                     { user: login_id, doctor: user._id },
    //                     { user: user._id, doctor: login_id }
    //                 ]
    //             }).sort({ created_at: -1 });

    //             const userId = user._id;
    //             console.log("🚀 ~ userDetailPromises ~ userId:", userId)
    //             console.log("🚀 ~ userDetailPromises ~ login_id:", login_id)

    //             const userDetailInfo = await UserMaster.findOne({ _id: userId }).select('profile fname lname');

    //             const unreadMSGCount = await messageMaster.find({
    //                 receiver_id: ObjectId(login_id),
    //                 seen: false
    //             });

    //             const appo_id = findconsultation ? findconsultation._id : "";

    //             if (userDetailInfo != null && userDetailInfo._id != login_id) {
    //                 return {
    //                     userDetail: userDetailInfo,
    //                     consultation_id: appo_id,
    //                     unreadCount: unreadMSGCount.length,
    //                     latestMessage: user.latestMessage
    //                 };
    //             }
    //         });

    //         const userDetail = await Promise.all(userDetailPromises);
    //         const filteredUserDetail = userDetail.filter(user => user !== null);

    //         filteredUserDetail.sort((a, b) => new Date(b.latestMessage.created_at) - new Date(a.latestMessage.created_at));

    //         socket.emit('setChatUserList', { resData: filteredUserDetail });
    //     } catch (error) {
    //         let errorMsg = error.message ?? "Something went wrong";
    //         console.log('errorMsg-setChatUserList->>', errorMsg);
    //         socket.emit("setChatUserList", errorMsg);
    //     }
    // });

    socket.on('getChatUserList', async (login_id) => {
        try {
            if (!socket.rooms.hasOwnProperty(login_id.toString())) {
                socket.join(login_id.toString());
            }

            const result = await messageMaster.aggregate([
                {
                    $match: {
                        $or: [
                            { sender_id: ObjectId(login_id) },
                            { receiver_id: ObjectId(login_id) }
                        ],
                    }
                },
                {
                    $sort: { created_at: -1 }
                },
                {
                    $group: {
                        _id: {
                            $cond: [
                                { $eq: ["$sender_id", ObjectId(login_id)] },
                                "$receiver_id",
                                "$sender_id"
                            ]
                        },
                        latestMessage: { $first: "$$ROOT" }
                    }
                },
                {
                    $lookup: {
                        from: "message_master",
                        let: { userId: "$_id" },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $and: [
                                            { $eq: ["$sender_id", "$$userId"] },
                                            { $eq: ["$receiver_id", ObjectId(login_id)] },
                                            { $eq: ["$seen", false] }
                                        ]
                                    }
                                }
                            },
                            {
                                $count: "unreadCount"
                            }
                        ],
                        as: "unreadMessages"
                    }
                },
                {
                    $addFields: {
                        unreadCount: { $arrayElemAt: ["$unreadMessages.unreadCount", 0] }
                    }
                },
                {
                    $lookup: {
                        from: "user_master",
                        localField: "_id",
                        foreignField: "_id",
                        as: "userDetail"
                    }
                },
                {
                    $unwind: "$userDetail"
                },
                {
                    $lookup: {
                        from: "consultation_master",
                        let: { userId: "$_id" },
                        pipeline: [
                            {
                                $match: {
                                    $expr: {
                                        $or: [
                                            { $and: [{ $eq: ["$user", ObjectId(login_id)] }, { $eq: ["$doctor", "$$userId"] }] },
                                            { $and: [{ $eq: ["$user", "$$userId"] }, { $eq: ["$doctor", ObjectId(login_id)] }] }
                                        ]
                                    }
                                }
                            },
                            { $sort: { created_at: -1 } },
                            { $limit: 1 }
                        ],
                        as: "consultation"
                    }
                },
                {
                    $addFields: {
                        consultation_id: { $arrayElemAt: ["$consultation._id", 0] }
                    }
                },
                {
                    $project: {
                        userDetail: {
                            profile: 1, fname: 1, lname: 1, is_suspended: 1, is_hide: 1, deleted_at: 1
                        },
                        consultation_id: 1,
                        unreadCount: { $ifNull: ["$unreadCount", 0] },
                        latestMessage: 1,

                    }
                }
            ]);

            // console.log(result,"==========");

            const filteredUserDetail = result.filter(user => user.userDetail && user.userDetail._id !== login_id);

            filteredUserDetail.sort((a, b) => new Date(b.latestMessage.created_at) - new Date(a.latestMessage.created_at));

            socket.emit('setChatUserList', { resData: filteredUserDetail });
        } catch (error) {
            let errorMsg = error.message ?? "Something went wrong";
            console.log('errorMsg-setChatUserList->>', error);
            socket.emit("setChatUserList", errorMsg);
        }
    });

    socket.on('getMessageList', async (login_id, receiver_id) => {
        try {
            if (!socket.rooms.hasOwnProperty(receiver_id.toString())) {
                socket.join(receiver_id.toString());
            }

            await messageMaster.updateMany({ sender_id: receiver_id, receiver_id: login_id, seen: false }, { seen: true }, { new: true })
            const result = await messageMaster.aggregate([
                {
                    $match: {
                        $or: [{ $and: [{ sender_id: ObjectId(login_id) }, { receiver_id: ObjectId(receiver_id) }] }, { $and: [{ sender_id: ObjectId(receiver_id) }, { receiver_id: ObjectId(login_id) }] }]
                    }
                },
                { $sort: { created_at: 1 } }
            ])
            console.log("🚀 ~ socket.on ~ result:", result)
            socket.emit('setMessageList', { resData: result });
        } catch (error) {
            let errorMsg = error.message ?? "Somethig went wrong";
            console.log('errorMsg-setMessageList->>', error);
            socket.emit("setMessageList", errorMsg);
        }
    })

    socket.on('disconnect', () => {
        console.log("disconnected user", socket.id)
    })
})

/* STRIPE WEEBHOOK */
const commonConfig = require("./config/common.config");
const UserMaster = require('./models/UserMaster');
const AdminMaster = require('./models/AdminMaster');
const ConsultationMaster = require('./models/ConsultationMaster');
const NotificationMaster = require('./models/NotificationMaster');
const TransactionMaster = require('./models/TransactionMaster');
const OrderMaster = require('./models/OrderMaster');
const jwt = require('jsonwebtoken');
const CartMaster = require('./models/CartMaster');
const veterinarianController = require('./controllers/veterinarian.controller');

const decryptData = (key) => {
    return jwt.verify(key, commonConfig.jwt.secret)
}
global.STRIPE_SECRET_KEY = decryptData(commonConfig.STRIPE_SECRET_KEY)
global.STRIPE_PUBLIC_KEY = decryptData(commonConfig.STRIPE_PUBLIC_KEY)

// console.log("🚀 ~ STRIPE_SECRET_KEY:", STRIPE_SECRET_KEY)
// console.log("🚀 ~ STRIPE_PUBLIC_KEY:", STRIPE_PUBLIC_KEY)

const stripe = require('stripe')(STRIPE_SECRET_KEY);
var userProfile;
app.get('/error', (req, res) => res.send("error logging in"));

app.get("/stripe_payment", (req, res) => {
    const { clientSecret } = req.query;
    const html = `<!DOCTYPE html>
            <html lang="en">
            <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>VETZY Payment</title>
            <style>
                /* Variables */
                * {
                    box-sizing: border-box;
                }

                body {
                    font-family: -apple-system, BlinkMacSystemFont, sans-serif;
                    font-size: 16px;
                    -webkit-font-smoothing: antialiased;
                    display: flex;
                    justify-content: center;
                    align-content: center;
                    height:full;
                    max-height: 100vh;
                    width: 100vw;
                    margin:0;
                }

                form {
                    width: 30vw;
                    min-width: 500px;
                    align-self: center;
                    box-shadow: 0px 0px 0px 0.5px rgba(50, 50, 93, 0.1),
                    0px 2px 5px 0px rgba(50, 50, 93, 0.1), 0px 1px 1.5px 0px rgba(0, 0, 0, 0.07);
                    border-radius: 7px;
                    padding: 20px;
                }

                .hidden {
                    display: none;
                }

                #payment-element {
                    margin-bottom: 24px;
                }

                /* Buttons and links */
                button {
                    background: #5469d4;
                    font-family: Arial, sans-serif;
                    color: #ffffff;
                    border-radius: 4px;
                    border: 0;
                    padding: 12px 16px;
                    font-size: 16px;
                    font-weight: 600;
                    cursor: pointer;
                    display: block;
                    transition: all 0.2s ease;
                    box-shadow: 0px 4px 5.5px 0px rgba(0, 0, 0, 0.07);
                    width: 100%;
                }
                button:hover {
                    filter: contrast(115%);
                }
                button:disabled {
                    opacity: 0.5;
                    cursor: default;
                }

                /* spinner/processing state, errors */
                .spinner,
                .spinner:before,
                .spinner:after {
                    border-radius: 50%;
                }
                .spinner {
                    color: #ffffff;
                    font-size: 22px;
                    text-indent: -99999px;
                    margin: 0px auto;
                    position: relative;
                    width: 20px;
                    height: 20px;
                    box-shadow: inset 0 0 0 2px;
                    -webkit-transform: translateZ(0);
                    -ms-transform: translateZ(0);
                    transform: translateZ(0);
                }
                .spinner:before,
                .spinner:after {
                    position: absolute;
                    content: "";
                }
                .spinner:before {
                    width: 10.4px;
                    height: 20.4px;
                    background: #5469d4;
                    border-radius: 20.4px 0 0 20.4px;
                    top: -0.2px;
                    left: -0.2px;
                    -webkit-transform-origin: 10.4px 10.2px;
                    transform-origin: 10.4px 10.2px;
                    -webkit-animation: loading 2s infinite ease 1.5s;
                    animation: loading 2s infinite ease 1.5s;
                }
                .spinner:after {
                    width: 10.4px;
                    height: 10.2px;
                    background: #5469d4;
                    border-radius: 0 10.2px 10.2px 0;
                    top: -0.1px;
                    left: 10.2px;
                    -webkit-transform-origin: 0px 10.2px;
                    transform-origin: 0px 10.2px;
                    -webkit-animation: loading 2s infinite ease;
                    animation: loading 2s infinite ease;
                }

                @-webkit-keyframes loading {
                    0% {
                        -webkit-transform: rotate(0deg);
                        transform: rotate(0deg);
                    }
                    100% {
                        -webkit-transform: rotate(360deg);
                        transform: rotate(360deg);
                    }
                }
                @keyframes loading {
                    0% {
                        -webkit-transform: rotate(0deg);
                        transform: rotate(0deg);
                    }
                    100% {
                        -webkit-transform: rotate(360deg);
                        transform: rotate(360deg);
                    }
                }

                @media only screen and (max-width: 600px) {
                    form {
                        width: 100vw;
                        min-width: initial;
                    }
                }
                #powered-by-stripe {
                    text-align: center;
                    color: #666;
                }
                #powered-by-stripe> p{
                    display: flex;
                    justify-content: center;
                    align-items: center;
                    margin:0 auto;
                    font-size: 20px;
                 }
            </style>
            <script src="https://js.stripe.com/v3/"></script>
            </head>
            <body>

           <form id="payment-form" class="dash_div " style="margin:0 auto ; box-shadow:none !important">
                <center>
  <h1 style="color: white; padding: 20px 0;">
    <img src='/public/images/rounded_icon.png' alt="image" style="max-width:100% ; height:100px"> 
  </h1>
</center>

                <div id="payment-element">
                    <!--Stripe.js injects the Payment Element-->
                </div>
                <button id="submit">
                    <div class="spinner hidden" id="spinner"></div>
                    <span id="button-text">Pay now </span>
                </button>
                <br>
                <button onclick="cancelPayment()" type="button">
                    <span>Cancel payment</span>
                </button>
                

            </form>

            <script type="text/javascript">
                // This is your test publishable API key.
                const stripe = Stripe("`+ STRIPE_PUBLIC_KEY + `");

                let elements;


                    document
                    .querySelector("#payment-form")
                    .addEventListener("submit", handleSubmit);


                let emailAddress = null;
                let returnUrl = "`+ global.APP_URL + `/success";
                console.log(returnUrl)
                // Fetches a payment intent and captures the client secret
                initialize('`+ clientSecret + `');
                async function initialize(clientSecret) {
                    elements = stripe.elements({ clientSecret });

                    // const linkAuthenticationElement = elements.create("linkAuthentication");
                    // linkAuthenticationElement.mount("#link-authentication-element");

                    const paymentElementOptions = {
                        layout: "tabs",
                    };

                    const paymentElement = elements.create("payment", paymentElementOptions);
                    paymentElement.mount("#payment-element");
                }

                async function cancelPayment(){
                    var message = '{"ResponseCode":0,"ResponseMsg":"Something went wrong, Please try again.","Result":"False"}';
                    Print.postMessage(message);
                }

                async function handleSubmit(e) {
                    e.preventDefault();
                    setLoading(true);

                    const { error } = await stripe.confirmPayment({
                        elements,
                        confirmParams: {
                            // Make sure to change this to your payment completion page
                            return_url: returnUrl,
                            receipt_email: emailAddress,
                        },
                    });
                    console.log(error);

                    if (error.type === "card_error" || error.type === "validation_error") {
                        alert(error.message)
                    } else {
                        alert("An unexpected error occurred.")
                    }

                    setLoading(false);
                }

                // Fetches the payment intent status after payment submission
                async function retrievePayIntent() {
                    const clientSecret = new URLSearchParams(window.location.search).get(
                        "payment_intent_client_secret"
                    );

                    if (!clientSecret) {
                        return;
                    }

                    const { paymentIntent } = await stripe.retrievePaymentIntent(clientSecret);
                    return paymentIntent;
                }

                // Show a spinner on payment submission
                function setLoading(isLoading) {
                    if (isLoading) {
                        // Disable the button and show a spinner
                        document.querySelector("#submit").disabled = true;
                        document.querySelector("#spinner").classList.remove("hidden");
                        document.querySelector("#button-text").classList.add("hidden");
                    } else {
                        document.querySelector("#submit").disabled = false;
                        document.querySelector("#spinner").classList.add("hidden");
                        document.querySelector("#button-text").classList.remove("hidden");
                    }
                }

            </script>


            </body>
            </html>

    `;

    res.send(html);
});

// Define a route for the success page
app.get('/success', (req, res) => {
    res.sendFile(__dirname + '/public/success.html');
});

app.post('/webhook_stripe', async (req, res) => {
    try {
        let event = req.body;
        console.log('stripe webhook event', event.type);

        // Handle the event
        switch (event.type) {
            case 'payment_intent.succeeded':
                const paymentIntent = event.data.object;

                // console.log("======event.log======", event.data);
                // console.log("Success-meta->>", paymentIntent.metadata);
                const { order_id, consultation_ids } = paymentIntent.metadata;

                let AdminDetail = await AdminMaster.findOne({ email: commonConfig.AdminEmail });
                // console.log("order_id==========="+order_id);
                if (order_id) {
                    let OrderDetail = await OrderMaster.findOneAndUpdate({ _id: order_id }, { payment: "success" }, { new: true });

                    // console.log("OrderDetail==========="+OrderDetail);




                    if (OrderDetail) {
                        for (var i = 0; i < OrderDetail.product_arr.length; i++) {
                            let buyQuantity = OrderDetail.product_arr.at(i).quantity
                            let productDetail = await ProductMaster.findOne({ _id: OrderDetail.product_arr.at(i).product_id })
                            if (buyQuantity > productDetail.quantity) {
                                return res.send(formatter.formatResponse({}, 0, "out_of_stock", false))
                            }
                            let remainQuantity = productDetail.quantity - buyQuantity

                            await ProductMaster.findByIdAndUpdate({ _id: OrderDetail.product_arr.at(i).product_id }, { quantity: remainQuantity })
                        }
                        if (AdminDetail) {
                            AdminDetail.wallet += Number(OrderDetail?.total);
                            await AdminDetail.save();
                        }
                        await CartMaster.findOneAndDelete({ user: OrderDetail.user })
                    }

                    // const productPayment = new TransactionMaster({
                    //     user_id: OrderDetail.user,
                    //     receiver_id: commonConfig.AdminId,
                    //     transaction_id: event.data.object.id,
                    //     price: OrderDetail.total,
                    //     msg: `Product purchase payment`
                    // }).save();
                } else if (consultation_ids) {
                    let Appo_ids = consultation_ids.split(",");

                    const updatePaymentAndNotify = async (ids) => {
                        const paymentStatusUpdate = await ConsultationMaster.findOneAndUpdate(
                            { _id: ids },
                            { payment: "success", chargeId: paymentIntent.latest_charge },
                            { new: true }
                        );

                        if (paymentStatusUpdate) {
                            let doctorDetails = await UserMaster.findOne({ _id: paymentStatusUpdate.doctor });

                            if (doctorDetails) {
                                await NotificationMaster.create({
                                    sender_id: paymentStatusUpdate.user,
                                    receiver_id: doctorDetails._id,
                                    pet_id: paymentStatusUpdate.pet,
                                    consultation_id: paymentStatusUpdate._id,
                                    msg: `New Instant Consultation Booked`,
                                    msg_type: "text",
                                    appointment_type: paymentStatusUpdate.appointment_type
                                });

                                if (doctorDetails.device_token) {
                                    await sendFirebaseNotification(doctorDetails.device_token, "VETZY", `New Instant Consultation Booked`);
                                }
                            }
                        }
                    };

                    await Promise.all(Appo_ids.map(updatePaymentAndNotify));
                } else {
                    const { consultation_id, user_id, doctor_id, pet_id, device_token } = paymentIntent.metadata;
                    const paymentStatusUpdate = await ConsultationMaster.findOneAndUpdate({ _id: consultation_id }, { payment: "success", chargeId: paymentIntent.latest_charge }, { new: true });

                    if (paymentStatusUpdate && paymentStatusUpdate.appointment_type == "Scheduled") {
                        let DoctorDetail = await UserMaster.findOne({ _id: doctor_id, deleted_at: null });
                        let doctorPrice;
                        if (DoctorDetail) {
                            DoctorDetail.wallet += DoctorDetail.price * (1 - (process.env.admin_charge / 100)).toFixed(2);
                            await DoctorDetail.save();
                            doctorPrice = DoctorDetail.price;
                        }
                        if (AdminDetail) {
                            AdminDetail.wallet += DoctorDetail.price;

                            // const adminChargeAddWallet = new TransactionMaster({
                            //     user_id: user_id,
                            //     receiver_id: doctor_id,
                            //     transaction_id: event.data.object.id,
                            //     price: DoctorDetail.price - DoctorDetail.price * (1 - (process.env.admin_charge / 100)),
                            //     msg: `Consultation admin charge`
                            // }).save();

                            await AdminDetail.save();
                        }

                        await messageMaster({
                            sender_id: doctor_id,
                            receiver_id: user_id,
                            msg: `New Scheduled Consultation Booked for ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}`,
                            msg_type: 'Scheduled Consultation',
                            seen: false
                        }).save();

                        const messageSend = new NotificationMaster({
                            sender_id: user_id,
                            receiver_id: doctor_id,
                            pet_id: pet_id,
                            consultation_id: consultation_id,
                            msg: `New Scheduled Consultation Booked for ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}`,
                            msg_type: "text",
                            appointment_type: paymentStatusUpdate.appointment_type
                        })
                        await messageSend.save()
                        if (device_token) await sendFirebaseNotification(device_token, "VETZY", `New Scheduled Consultation Booked for ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}`);


                        const transaction = new TransactionMaster({
                            user_id: user_id,
                            receiver_id: doctor_id,
                            transaction_id: event.data.object.id,
                            price: doctorPrice,
                            msg: `Consultation fees`
                        });
                        await transaction.save();
                    }
                    // if (paymentStatusUpdate.appointment_type == "Instant") {
                    //     const messageSend = new NotificationMaster({
                    //         sender_id: user_id,
                    //         receiver_id: doctor_id,
                    //         pet_id: pet_id,
                    //         consultation_id: consultation_id,
                    //         msg: `Has sent you an Instant appointment request. Please take action on the request`,
                    //         msg_type: "text",
                    //         appointment_type: paymentStatusUpdate.appointment_type
                    //     })
                    //     await messageSend.save()
                    //     if (device_token) await sendFirebaseNotification(device_token, "VETZY", `New Instant Consultation Booked`);
                    // }

                }
                res.send("payment success!");
                break;

            case 'payment_intent.payment_failed':
                const paymentFailedIntent = event.data.object;
                console.log("Failed-->>", paymentFailedIntent);
                res.send("payment failed!");
                break;

            // Add more event types as needed
            default:
                console.log(`Unhandled event type: ${event.type}`);
                res.send("payment default!");
                break;
        }
    } catch (error) {
        console.error('Error handling webhook:', error);
        res.status(500).send("Internal Server Error");
    }
});

// app.post('/webhook_stripe', async (req, res) => {
//     try {
//         let event = req.body;
//         console.log('stripe webhook event', event.type);

//         // Handle the event
//         switch (event.type) {
//             case 'payment_intent.succeeded':
//                 const paymentIntent = event.data.object;
//                 console.log("Success-meta->>", paymentIntent.metadata);
//                 const { order_id } = paymentIntent.metadata;

//                 let AdminDetail = await AdminMaster.findOne({ email: commonConfig.AdminEmail });

//                 if (order_id) {
//                     const OrderDetail = await OrderMaster.findOne({ _id: order_id });
//                     AdminDetail.wallet += OrderDetail.total;
//                     await AdminDetail.save();
//                     // const productPayment = new TransactionMaster({
//                     //     user_id: OrderDetail.user,
//                     //     receiver_id: commonConfig.AdminId,
//                     //     transaction_id: event.data.object.id,
//                     //     price: OrderDetail.total,
//                     //     msg: `Product purchase payment`
//                     // }).save();
//                 } else {
//                     const { consultation_id, user_id, doctor_id, pet_id, device_token } = paymentIntent.metadata;
//                     const paymentStatusUpdate = await ConsultationMaster.findOneAndUpdate({ _id: consultation_id }, { payment: "success" }, { new: true });

//                     if (paymentStatusUpdate.appointment_type == "Scheduled") {
//                         let DoctorDetail = await UserMaster.findOne({ _id: doctor_id, deleted_at: null });

//                         DoctorDetail.wallet += DoctorDetail.price * (1 - (process.env.admin_charge / 100));
//                         AdminDetail.wallet += Number(DoctorDetail.price) - Number(DoctorDetail.price * (1 - (process.env.admin_charge / 100)));

//                         // const adminChargeAddWallet = new TransactionMaster({
//                         //     user_id: user_id,
//                         //     receiver_id: doctor_id,
//                         //     transaction_id: event.data.object.id,
//                         //     price: DoctorDetail.price - DoctorDetail.price * (1 - (process.env.admin_charge / 100)),
//                         //     msg: `Consultation admin charge`
//                         // }).save();

//                         await AdminDetail.save();
//                         await DoctorDetail.save();

//                         const messageSend = new NotificationMaster({
//                             sender_id: user_id,
//                             receiver_id: doctor_id,
//                             pet_id: pet_id,
//                             consultation_id: consultation_id,
//                             msg: `Has sent you an appointment request to visit on ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}. Please take action on the request.`,
//                             msg_type: "text",
//                             appointment_type: paymentStatusUpdate.appointment_type
//                         })
//                         await messageSend.save()
//                         if (device_token) await sendFirebaseNotification(device_token, "VETZY", `New Scheduled Consultation Booked for ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}`);


//                         const transaction = new TransactionMaster({
//                             user_id: user_id,
//                             receiver_id: doctor_id,
//                             transaction_id: event.data.object.id,
//                             price: DoctorDetail.price,
//                             msg: `Consultation fees`
//                         });
//                         await transaction.save();
//                     }
//                     // if (paymentStatusUpdate.appointment_type == "Instant") {
//                         //     const messageSend = new NotificationMaster({
//                         //         sender_id: user_id,
//                         //         receiver_id: doctor_id,
//                         //         pet_id: pet_id,
//                         //         consultation_id: consultation_id,
//                         //         msg: `Has sent you an Instant appointment request. Please take action on the request`,
//                         //         msg_type: "text",
//                         //         appointment_type: paymentStatusUpdate.appointment_type
//                         //     })
//                         //     await messageSend.save()
//                         //     if (device_token) await sendFirebaseNotification(device_token, "VETZY", `New Instant Consultation Booked`);
//                     // }

//                 }
//                 res.send("payment success!");
//                 break;

//             case 'payment_intent.payment_failed':
//                 const paymentFailedIntent = event.data.object;
//                 console.log("Failed-->>", paymentFailedIntent);
//                 res.send("payment failed!");
//                 break;

//             // Add more event types as needed
//             default:
//                 console.log(`Unhandled event type: ${event.type}`);
//                 res.send("payment default!");
//                 break;
//         }
//     } catch (error) {
//         console.error('Error handling webhook:', error);
//         res.status(500).send("Internal Server Error");
//     }
// });

// app.post('/webhook_stripe', async (req, res) => {
//     try {
//         let event = req.body;
//         console.log('stripe webhook event', event.type);

//         // Handle the event
//         switch (event.type) {
//             case 'payment_intent.succeeded':
//                 const paymentIntent = event.data.object;
//                 console.log("Success-meta->>", paymentIntent.metadata);
//                 const { order_id } = paymentIntent.metadata;

//                 let AdminDetail = await AdminMaster.findOne({ email: commonConfig.AdminEmail });

//                 if (order_id) {
//                     const OrderDetail = await OrderMaster.findOne({ _id: order_id });
//                     AdminDetail.wallet += OrderDetail.total;
//                     await AdminDetail.save();
//                     // const productPayment = new TransactionMaster({
//                     //     user_id: OrderDetail.user,
//                     //     receiver_id: commonConfig.AdminId,
//                     //     transaction_id: event.data.object.id,
//                     //     price: OrderDetail.total,
//                     //     msg: `Product purchase payment`
//                     // }).save();
//                 } else {
//                     const { consultation_id, user_id, doctor_id, pet_id, device_token } = paymentIntent.metadata;
//                     const paymentStatusUpdate = await ConsultationMaster.findOneAndUpdate({ _id: consultation_id }, { payment: "success" }, { new: true });

//                     let DoctorDetail = await UserMaster.findOne({ _id: doctor_id, deleted_at: null });

//                     DoctorDetail.wallet += DoctorDetail.price * (1 - (process.env.admin_charge / 100));
//                     AdminDetail.wallet += Number(DoctorDetail.price) - Number(DoctorDetail.price * (1 - (process.env.admin_charge / 100)));

//                     // const adminChargeAddWallet = new TransactionMaster({
//                     //     user_id: user_id,
//                     //     receiver_id: doctor_id,
//                     //     transaction_id: event.data.object.id,
//                     //     price: DoctorDetail.price - DoctorDetail.price * (1 - (process.env.admin_charge / 100)),
//                     //     msg: `Consultation admin charge`
//                     // }).save();

//                     await AdminDetail.save();
//                     await DoctorDetail.save();

//                     // const msg = paymentStatusUpdate.appointment_type === "Scheduled"
//                     //     ? `Has sent you an appointment request to visit on ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}. Please take action on the request.`
//                     //     : `Has sent you an Instant appointment request. Please take action on the request`;

//                     // const messageSend = new NotificationMaster({
//                     //     sender_id: user_id,
//                     //     receiver_id: doctor_id,
//                     //     pet_id: pet,
//                     //     consultation_id: consultation_id,
//                     //     msg,
//                     //     msg_type: "text",
//                     //     appointment_type: paymentStatusUpdate.appointment_type
//                     // });
//                     // await messageSend.save();
//                     if (paymentStatusUpdate.appointment_type == "Scheduled") {
//                         const messageSend = new NotificationMaster({
//                             sender_id: user_id,
//                             receiver_id: doctor_id,
//                             pet_id: pet_id,
//                             consultation_id: consultation_id,
//                             msg: `Has sent you an appointment request to visit on ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}. Please take action on the request.`,
//                             msg_type: "text",
//                             appointment_type: paymentStatusUpdate.appointment_type
//                         })
//                         await messageSend.save()
//                         if (device_token) await sendFirebaseNotification(device_token, "VETZY", `New Scheduled Consultation Booked for ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}`);
//                     }
//                     if (paymentStatusUpdate.appointment_type == "Instant") {
//                         const messageSend = new NotificationMaster({
//                             sender_id: user_id,
//                             receiver_id: doctor_id,
//                             pet_id: pet_id,
//                             consultation_id: consultation_id,
//                             msg: `Has sent you an Instant appointment request. Please take action on the request`,
//                             msg_type: "text",
//                             appointment_type: paymentStatusUpdate.appointment_type
//                         })
//                         await messageSend.save()
//                         if (device_token) await sendFirebaseNotification(device_token, "VETZY", `New Instant Consultation Booked`);
//                     }

//                     // if (device_token) {
//                     //     await sendFirebaseNotification(device_token, "VETZY", `New consultation booked at ${moment(paymentStatusUpdate.schedule_time).format('MMMM Do YYYY, h:mm:ss a')}`);
//                     // }

//                     const transaction = new TransactionMaster({
//                         user_id: user_id,
//                         receiver_id: doctor_id,
//                         transaction_id: event.data.object.id,
//                         price: DoctorDetail.price,
//                         msg: `Consultation fees`
//                     });
//                     await transaction.save();
//                 }
//                 res.send("payment success!");
//                 break;

//             case 'payment_intent.payment_failed':
//                 const paymentFailedIntent = event.data.object;
//                 console.log("Failed-->>", paymentFailedIntent);
//                 res.send("payment failed!");
//                 break;

//             // Add more event types as needed
//             default:
//                 console.log(`Unhandled event type: ${event.type}`);
//                 res.send("payment default!");
//                 break;
//         }
//     } catch (error) {
//         console.error('Error handling webhook:', error);
//         res.status(500).send("Internal Server Error");
//     }
// });