const adminController = {};
const Q = require('q');
const fs = require('fs')
const config = require('../config/common.config');
const jwt = require('jsonwebtoken');
const sendMail = require("../utils/sendMail");
const { contactUsMail, registerMail, forgotPasswordMail } = require("../utils/ContentProvider");

const publicPath = global.basedir + "/public/";
const AdminMaster = require('../models/AdminMaster');
const UserMaster = require('../models/UserMaster');
const ProductMaster = require('../models/ProductMaster')
const PetTypeMaster = require('../models/PetTypeMaster');
const pricribeMaster = require('../models/PriscribeMaster')
const CategoryMaster = require('../models/CategoryMaster');
const BreedAndSpeciesMaster = require('../models/BreedAndSpeciesMaster');
const OrderMaster = require('../models/OrderMaster');
const PetMaster = require('../models/PetMaster');
const ConsultationMaster = require('../models/ConsultationMaster');
const MessageMaster = require('../models/MessageMaster');
const { returnPayment, payoutStripe } = require('./payment.controller');
const mongoose = require("mongoose");
const TransactionMaster = require('../models/TransactionMaster');
const PayoutHistoryMaster = require('../models/PayoutHistoryMaster');
const NotificationMaster = require('../models/NotificationMaster');
const { sendFirebaseNotification } = require('../utils/notification');

(async () => {
    try {
        let findAdmin = await AdminMaster.findOne({ email: "admin@gmail.com" })
        if (!findAdmin) {
            new AdminMaster({
                fname: "admin",
                lname: "admin",
                email: "admin@gmail.com",
                password: "12345678",
            }).save()
            return
        }
    } catch (error) {
        return error
    }
})();

adminController.login = async (req, res) => {
    let deferred = Q.defer()
    const { email, password } = req.body
    try {
        let findAdmin = await AdminMaster.findOne({ email: email.toLowerCase() })
        if (findAdmin) {
            if (password == findAdmin.password) {
                let token = jwt.sign({ _id: findAdmin._id }, config.jwt.secret, { expiresIn: config.jwt.token_expiry })
                findAdmin.token = token
                deferred.resolve(findAdmin);
            } else {
                deferred.reject("incorrect_password")
            }
        } else {
            deferred.reject("incorrect_email")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.forgotPassword = async (req, res) => {
    let deferred = Q.defer()
    let { email } = req.body;
    try {
        let randomOTP = Math.floor(Math.random() * (9999 - 1000 + 1)) + 1000
        let findAdmin = await AdminMaster.findOne({ email: email.toLowerCase() })
        if (!findAdmin) {
            deferred.reject("incorrect_email")
            return deferred.promise;
        } else {
            let updateOTP = await AdminMaster.findOneAndUpdate({ email: email.toLowerCase() }, { otp: randomOTP }, { new: true })
            if (updateOTP) {
                let mailContent = await forgotPasswordMail(findAdmin.name, randomOTP);
                sendMail(email.toLowerCase(), "Forgot Password Mail", mailContent);
                deferred.resolve(updateOTP);
            }
        }
    } catch (error) {
        deferred.reject(error.message);
    }
    return deferred.promise;
}

adminController.verifyOTP = async (req, res) => {
    let deferred = Q.defer()
    let { otp, email } = req.body
    try {
        let findAdmin = await AdminMaster.findOne({ email: email.toLowerCase() })
        if (!findAdmin) {
            deferred.reject("incorrect_email")
            return deferred.promise;
        }
        if (otp == findAdmin.otp) {
            if (findAdmin.otp != null) {
                await AdminMaster.findOneAndUpdate({ email: email.toLowerCase() }, { otp: null }, { new: true })
                deferred.resolve({});
            }
        } else {
            deferred.reject("invalid_otp")
        }
    } catch (error) {
        deferred.reject(error.message);
    }
    return deferred.promise
}

adminController.resetPassword = async (req, res) => {
    let deferred = Q.defer();
    let { email, new_password } = req.body;
    try {
        let adminDetails = await AdminMaster.findOne({ email: email });

        if (adminDetails != null) {
            let userResponseObj = await AdminMaster.findOneAndUpdate({ email: email }, { password: new_password }, { new: true });
            deferred.resolve(userResponseObj);
        }
        else {
            deferred.reject("invalid_user_id");
        }
    } catch (error) {
        deferred.reject(error.message);
    }
    return deferred.promise;
}

adminController.userList = async (req, res) => {
    let deferred = Q.defer();
    let { search, user_type, page = 1, limit = 10 } = req.body;
    try {

        // console.log("page========="+page);
        // console.log("limit========="+limit);

        let searchPattern = new RegExp(search, 'i');
        let skip = (page - 1) * limit;
        let filter = { deleted_at: null };
        if (user_type) {
            filter = { user_type: user_type ? user_type : "", deleted_at: null };
        }


        let findUser = await UserMaster.aggregate([
            {
                $match: {
                    ...filter,
                    $or: [
                        { fname: searchPattern },
                        { lname: searchPattern },
                        { pet_type: searchPattern },
                        { email: searchPattern },
                        { specialization: searchPattern },
                    ]
                }
            },
            {
                $lookup: {
                    from: 'consultation_master', // The name of the collection you want to join
                    let: { userId: '$_id' }, // Local field for join
                    pipeline: [
                        {
                            $match: {
                                $expr: {
                                    $and: [
                                        {
                                            $or: [
                                                { $eq: ['$doctor', '$$userId'] },
                                                { $eq: ['$user', '$$userId'] }
                                            ]
                                        },
                                        { $in: ['$status', ["pending", "accept"]] },
                                        { $eq: ['$payment', 'success'] }
                                    ]
                                }
                            }
                        }
                    ],
                    as: 'appointments'
                }
            },
            {
                $addFields: {
                    hasAppointment: { $gt: [{ $size: '$appointments' }, 0] },
                    profile: {
                        $cond: {
                            if: { $and: [{ $ne: ['$profile', null] }, { $ne: ['$profile', ''] }] },
                            then: { $concat: [global.APP_URL + '/public/profile/', { $arrayElemAt: [{ $split: ['$profile', '/'] }, -1] }] },
                            else: '' // Return an empty string if profile is null or empty
                        }
                    }
                }
            },
            {
                $project: {
                    appointments: 0,
                    // Include all other fields (set to 1) or leave them out entirely to include by default
                }
            },
            { $sort: { _id: -1 } },
            { $skip: skip },
            { $limit: +limit }
        ]);


        let totalData = await UserMaster.countDocuments({
            ...filter,
            $or: [
                { fname: searchPattern },
                { lname: searchPattern },
                { pet_type: searchPattern },
                { email: searchPattern },
                { specialization: searchPattern },
            ]
        }).setOptions({ ignorePreHooks: true });

        // console.log("========totalData=======" + totalData);


        let result = {
            user_list: findUser,
            total_count: totalData
        };
        deferred.resolve(result);
    } catch (error) {
        deferred.reject(error.message);
    }
    return deferred.promise;
};

adminController.category = async (req, res) => {
    let deferred = Q.defer()
    const { category_id, name, catType } = req.body
    try {
        let exist = await CategoryMaster.findOne(({ name: name, cat_type: catType }))
        if (!exist) {
            let category_image = req.file ? req.file.filename : ""
            let categoryAdd = new CategoryMaster({
                category_id: (category_id == "" ? null : category_id),
                name: name,
                category_image: category_image,
                cat_type: catType
            })
            await categoryAdd.save()
            deferred.resolve(categoryAdd)
        } else {
            deferred.reject('category_already_added')
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.categoryList = async (req, res) => {
    let deferred = Q.defer()
    let { category_id, page = 1, limit = 10, search, catType } = req.body;
    try {
        let skip = (page - 1) * limit;
        let query = { cat_type: catType };
        if (category_id) {
            query.category_id = category_id;
        } else {
            query.category_id = null;
        }
        if (search) {
            query.name = { $regex: search, $options: 'i' };
        }

        let catList = await CategoryMaster.find(query).sort({ _id: -1 }).skip(skip).limit(limit);
        let totalData = await CategoryMaster.countDocuments(query);
        let result = {
            catList: catList,
            totalData: totalData
        }
        deferred.resolve(result)
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.categoryUpdate = async (req, res) => {
    let deferred = Q.defer()
    let { _id, category_id, name } = req.body
    try {
        if (category_id && _id) {
            let existCategoryName = await CategoryMaster.findOne({ _id: _id, category_id: { $ne: null } })

            if (existCategoryName) {
                if (existCategoryName.name == name) {
                    let catUpdate = await CategoryMaster.findOneAndUpdate({ _id: _id, category_id: category_id }, { name: name }, { new: true })
                    deferred.resolve(catUpdate)
                } else {
                    let existCategoryName = await CategoryMaster.findOne({ name: name, category_id: { $ne: null } })
                    if (existCategoryName) {
                        deferred.reject('category_exist')
                    } else {
                        let catUpdate = await CategoryMaster.findOneAndUpdate({ _id: _id, category_id: category_id }, { name: name }, { new: true })
                        deferred.resolve(catUpdate)
                    }
                }
            } else {
                deferred.reject('category_exist')
            }
        } else {
            let existCategoryName = await CategoryMaster.findOne({ _id: _id, category_id: null })

            // console.log("existCategoryName====" + existCategoryName);
            if (existCategoryName) {
                if (existCategoryName.name == name) {
                    let existingCat = await CategoryMaster.findOne({ _id: _id })
                    if (req.file) {
                        if (existingCat.category_image) fs.unlinkSync(publicPath + 'category/' + existingCat.category_image)
                        let catUpdate = await CategoryMaster.findOneAndUpdate({ _id: _id }, { name: name, category_image: req.file.filename }, { new: true })
                        deferred.resolve(catUpdate)
                    } else {
                        let catUpdate = await CategoryMaster.findOneAndUpdate({ _id: _id }, { name: name }, { new: true })
                        deferred.resolve(catUpdate)
                    }
                } else {
                    let existCategoryName = await CategoryMaster.findOne({ name: name, category_id: null })
                    if (existCategoryName) {
                        deferred.reject('category_exist')
                    } else {
                        let existingCat = await CategoryMaster.findOne({ _id: _id })
                        if (req.file) {
                            if (existingCat.category_image) fs.unlinkSync(publicPath + 'category/' + existingCat.category_image)
                            let catUpdate = await CategoryMaster.findOneAndUpdate({ _id: _id }, { name: name, category_image: req.file.filename }, { new: true })
                            deferred.resolve(catUpdate)
                        } else {
                            let catUpdate = await CategoryMaster.findOneAndUpdate({ _id: _id }, { name: name }, { new: true })
                            deferred.resolve(catUpdate)
                        }
                    }
                }
            } else {

                deferred.reject('category_exist')
            }
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.categoryDelete = async (req, res) => {
    let deferred = Q.defer()
    const { _id } = req.body
    try {
        let findCat = await CategoryMaster.findOne({ _id: _id })

        let productExist
        if (findCat.category_id == null) {
            productExist = await ProductMaster.findOne({ category_id: findCat._id })
        } else {
            productExist = await ProductMaster.findOne({ sub_category_id: findCat._id })
        }
        if (productExist) {
            deferred.reject('product_exist_in_category')
            return deferred.promise;
        }
        let deleteCat = await CategoryMaster.findOneAndDelete({ _id: _id })
        deferred.resolve(deleteCat)
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.product = async (req, res) => {
    let deferred = Q.defer()
    const { sub_category_id, category_id, name, description, price, sku, productType, quantity } = req.body
    try {
        // const category = await CategoryMaster.findOne(({ sub_category_id: sub_category_id }))
        let existSKU = await ProductMaster.findOne(({ sku: sku }))
        if (existSKU) {
            deferred.reject('SKU_unique')
            return deferred.promise;
        }
        let existProduct = await ProductMaster.findOne(({ name: name, sub_category_id: sub_category_id }))
        if (existProduct) {
            deferred.reject('product_already_added')
            return deferred.promise;
        }
        let product_image = req.file ? req.file.filename : "";
        let addProduct = new ProductMaster({
            category_id: category_id,
            sub_category_id: sub_category_id,
            name: name,
            description: description,
            price: price,
            product_image: product_image,
            sku: sku,
            productType: productType,
            quantity: quantity
        })
        await addProduct.save();
        deferred.resolve(addProduct)
    } catch (errorCode) {
        deferred.reject(errorCode)
    }
}

adminController.productUpdate = async (req, res) => {
    let deferred = Q.defer()
    const { product_id, name, description, price, category_id, sub_category_id, sku, quantity } = req.body
    try {
        let existSKU = await ProductMaster.findOne(({ sku: sku }))
        // console.log("existSKU==="+existSKU);
        // console.log("existSKU._id==="+existSKU._id);
        // console.log("product_id==="+product_id);
        if (existSKU && existSKU._id == product_id) {
            let existingProd = await ProductMaster.findOne({ _id: product_id })
            if (req.file) {
                if (existingProd.product_image) fs.unlink(publicPath + 'product/' + existingProd.product_image, (err) => {
                    if (err) console.log(err)
                })
                let prodUpdate = await ProductMaster.findOneAndUpdate({ _id: product_id }, { name: name, product_image: req.file.filename, price: price, description: description, category_id: category_id, sub_category_id: sub_category_id, sku: sku, quantity: quantity }, { new: true })
                deferred.resolve(prodUpdate)
            } else {
                let prodUpdate = await ProductMaster.findOneAndUpdate({ _id: product_id }, { name: name, price: price, description: description, category_id: category_id, sub_category_id: sub_category_id, sku: sku, quantity: quantity }, { new: true })


                deferred.resolve(prodUpdate)
            }
        } else {
            deferred.reject('SKU_unique')
            return deferred.promise;
        }

    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.productDelete = async (req, res) => {
    let deferred = Q.defer()
    const { product_id } = req.body
    try {
        let existingProd = await ProductMaster.findOneAndUpdate({ _id: product_id, deleted_at: null }, { deleted_at: new Date() }, { new: true })
        // if (existingProd.product_image) fs.unlinkSync(publicPath + 'product/' + existingProd.product_image)
        deferred.resolve({})
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}
adminController.productList = async (req, res) => {
    let deferred = Q.defer();
    const { sub_category_id, page = 1, limit = 10, search, productType } = req.body;
    const staticPath = APP_URL + "/public/product/";

    try {
        let matchStage = { deleted_at: null };
        if (search) {
            matchStage.$or = [
                { name: { $regex: search, $options: 'i' } },
                { description: { $regex: search, $options: 'i' } },
                { sku: { $regex: search, $options: 'i' } },
            ];
        }
        if (sub_category_id) {
            matchStage.sub_category_id = sub_category_id;
        }

        let skip = (page - 1) * limit;

        let prodList = await ProductMaster.aggregate([
            {
                $lookup: {
                    from: 'category_master',
                    localField: 'sub_category_id',
                    foreignField: '_id',
                    as: 'sub_category'
                }
            },
            // { $unwind: '$sub_category' },
            {
                $lookup: {
                    from: 'category_master',
                    localField: 'category_id',
                    foreignField: '_id',
                    as: 'category'
                }
            },
            // { $unwind: '$category' },
            {
                $match: {
                    ...matchStage, // existing match conditions
                    productType: productType, // replace with the actual product type value you want to filter by
                    deleted_at: null
                }
            },

            { $sort: { _id: -1 } },
            { $skip: skip },
            { $limit: limit },
        ]);

        prodList.forEach(product => {
            if (product.product_image) {
                product.product_image = staticPath + product.product_image;
            }
        });
        let forDoc = {
            ...matchStage, // Spread in your existing match conditions
            productType: productType // Replace with the actual product type value
        };
        let totalData = await ProductMaster.countDocuments(forDoc);

        let result = {
            prodList: prodList,
            totalData: totalData
        };
        deferred.resolve(result);
    } catch (error) {
        deferred.reject(error.message);
    }

    return deferred.promise;
};

adminController.createPetType = async (req, res) => {
    let deferred = Q.defer()
    const { pet_name } = req.body
    try {
        let pet_image = req.file ? req.file.filename : ""
        let pet_data = new PetTypeMaster({
            pet_name: pet_name.charAt(0).toUpperCase() + pet_name.slice(1), pet_image: pet_image
        })
        await pet_data.save()
        deferred.resolve(pet_data)
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.createPetBreedSpecies = async (req, res) => {
    let deferred = Q.defer()
    const { name, type, pet_type_id } = req.body
    try {
        let pet_data = new BreedAndSpeciesMaster({
            name, type, pet_type_id
        })
        await pet_data.save()
        deferred.resolve(pet_data)
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.updatePetType = async (req, res) => {
    let deferred = Q.defer()
    const { pet_name, pet_id } = req.body
    try {
        let findPetImage = await PetTypeMaster.findOne({ _id: pet_id })

        let update
        if (req.file) {
            let pet_image = req.file ? req.file.filename : findPetImage.pet_image
            if (findPetImage.pet_image) fs.unlinkSync(publicPath + 'petType/' + findPetImage.pet_image)
            update = await PetTypeMaster.findOneAndUpdate({ _id: pet_id }, { pet_name: pet_name.charAt(0).toUpperCase() + pet_name.slice(1), pet_image: pet_image }, { new: true })
        } else {
            update = await PetTypeMaster.findOneAndUpdate({ _id: pet_id }, { pet_name: pet_name.charAt(0).toUpperCase() + pet_name.slice(1) }, { new: true })
        }
        deferred.resolve(update)
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.deletePetType = async (req, res) => {
    let deferred = Q.defer()
    const { pet_id } = req.body
    try {
        let deletePetData = await PetTypeMaster.findOneAndDelete({ _id: pet_id })
        if (deletePetData.pet_image) fs.unlinkSync(publicPath + 'petType/' + deletePetData.pet_image)
        deferred.resolve({})
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.PetTypeList = async (req, res) => {
    let deferred = Q.defer()
    try {
        let petTypeList = await PetTypeMaster.find().sort({ _id: -1 })
        deferred.resolve(petTypeList)
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.prescriptionList = async (req, res) => {
    let deferred = Q.defer()
    let { page = 1, limit = 10, doctor_id, search = "" } = req.body;
    try {
        let skip = (page - 1) * limit;

        let totalRecords = await pricribeMaster.find({ doctor: doctor_id }).countDocuments();

        let filteredPrescriptions = await pricribeMaster.find({ doctor: doctor_id })
            .populate({
                path: 'user',
                options: { ignorePreHooks: true },
                match: {
                    $or: [
                        { fname: { $regex: search, $options: 'i' } },
                        { lname: { $regex: search, $options: 'i' } }
                    ]
                },
                select: 'fname lname profile'
            })
            .populate({
                path: 'pet_id',
                match: { name: { $regex: search, $options: 'i' } }
            })
            .populate({
                path: 'doctor',
                options: { ignorePreHooks: true },
                match: {
                    $or: [
                        { fname: { $regex: search, $options: 'i' } },
                        { lname: { $regex: search, $options: 'i' } }
                    ]
                },
                select: 'fname lname profile'
            })
            .populate({
                path: 'consultation',
                match: { description: { $regex: search, $options: 'i' } }
            })
            .populate({
                path: 'product_arr.product',
                model: 'product_master',
            })
            .sort({ _id: -1 })
            .skip(skip)
            .limit(limit);


        let prescriptionList = filteredPrescriptions.map(item => {
            if (item && item.user && item.user.profile) {
                item.user.profile = global.APP_URL + '/public/profile/' + item.user.profile.split('/').pop();
            }

            if (item && item.pet_id && item.pet_id.pet_profile) {
                item.pet_id.pet_profile = global.APP_URL + '/public/pet/' + item.pet_id.pet_profile.split('/').pop();
            }
            if (item && item.pet_id && item.pet_id.pet_document) {
                item.pet_id.pet_document = global.APP_URL + '/public/pet/' + item.pet_id.pet_document.split('/').pop();
            }
            if (item && item.doctor && item.doctor.profile) {
                item.doctor.profile = global.APP_URL + '/public/profile/' + item.doctor.profile.split('/').pop();
            }
            if (item && item.doctor && item.doctor.document) {
                item.doctor.document = global.APP_URL + '/public/profile/' + item.doctor.document.split('/').pop();
            }
            if (item && item.pet_id && item.pet_id.image_arr && item.pet_id.image_arr.length > 0) {
                item.pet_id.image_arr = item.pet_id.image_arr.map(productItem => {
                    if (productItem.image && productItem.image) {
                        productItem.image = global.APP_URL + '/public/pet/' + productItem.image.split('/').pop();
                    }
                    return productItem;
                });
            }

            return item;
        });


        deferred.resolve({ totalRecords: totalRecords || 0, prescriptionList })
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.editDoctorInfo = async (req, res) => {
    let deferred = Q.defer()
    let { doctor_id, phone, ccode } = req.body;
    try {
        let profile = req.file ? (req.file.filename ?? '') : '';
        req.body.profile = profile;

        // console.log("======profile===="+profile);

        if (profile == '') {
            delete req.body.profile;
        }
        else {
            let oldPath = publicPath + "profile/" + req.user.profile;
            fs.unlink(oldPath, (err) => {
                if (err) {
                    console.error(err);
                }
            })
        }

        let userResponseObj = await UserMaster.findOne({ user_type: "veterinarian", phone: phone, ccode: ccode }).setOptions({ ignorePreHooks: true });


        // console.log("userResponseObj._id===========" + userResponseObj._id);
        // console.log("doctor_id===========" + doctor_id);


        if (!userResponseObj || userResponseObj._id == doctor_id) {
            let userResponseObj = await UserMaster.findOneAndUpdate({ _id: doctor_id, user_type: "veterinarian" }, req.body, { new: true }).setOptions({ ignorePreHooks: true });
            if (userResponseObj) {
                deferred.resolve(userResponseObj)
            }
            else {
                deferred.reject("invalid_doctor_id")
            }
        } else {
            deferred.reject("already_register_phone")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.editUserInfo = async (req, res) => {
    let deferred = Q.defer()
    let { userId, phone, ccode } = req.body;
    try {
        let profile = req.file ? (req.file.filename ?? '') : '';
        req.body.profile = profile;
        if (profile == '') {
            delete req.body.profile;
        }
        else {
            let oldPath = publicPath + "profile/" + req.user.profile;
            fs.unlink(oldPath, (err) => {
                if (err) {
                    console.error(err);
                }
            })
        }
        let userResponseObj = await UserMaster.findOne({ user_type: "pet_parent", phone: phone, ccode: ccode }).setOptions({ ignorePreHooks: true });

        // console.log("userResponseObj============", userResponseObj);

        if (!userResponseObj || userResponseObj._id == userId) {
            let userResponseObj = await UserMaster.findOneAndUpdate({ _id: userId, user_type: "pet_parent" }, req.body, { new: true }).setOptions({ ignorePreHooks: true });
            if (userResponseObj) {
                deferred.resolve(userResponseObj)
            } else {
                deferred.reject("invalid_user_id")
            }
        } else {
            deferred.reject("already_register_phone")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.deleteDoctor = async (req, res) => {
    let deferred = Q.defer()
    let { doctor_id, reason } = req.body;
    try {
        let userResponseObj = await UserMaster.findOneAndUpdate({ _id: doctor_id, deleted_at: null }, { deleted_at: new Date(), reason: reason }, { new: true }).setOptions({ ignorePreHooks: true });
        if (userResponseObj) {
            if (userResponseObj.user_type == "pet_parent") {
                let totalAppo = await ConsultationMaster.find({ user: doctor_id, status: { $in: ["pending", "accept"] }, payment: "success" })
                totalAppo.map(async (consulation) => {
                    if (consulation.chargeId) {
                        let refund = await returnPayment(consulation.chargeId)
                        await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -consulation.fee } })
                        if (refund && refund.id) {
                            await TransactionMaster.create({
                                user_id: req.user._id,
                                receiver_id: doctor_id,
                                transaction_id: refund.id,
                                price: consulation.fee,
                                msg: "Refund Payment"
                            })
                        }
                    }
                    await MessageMaster.create({
                        sender_id: doctor_id == consulation.user ? consulation.doctor : consulation.user,
                        receiver_id: doctor_id,
                        msg: consulation._id,
                        msg_type: "Consultation Rejected"
                    });
                    await ConsultationMaster.updateOne({ _id: consulation._id }, { status: "reject" })
                });
                deferred.resolve('user_delete_success')
            } else {
                let totalAppo = await ConsultationMaster.find({ doctor: doctor_id, status: { $in: ["pending", "accept"] }, payment: "success" })
                totalAppo.map(async (consulation) => {
                    if (consulation.chargeId) {
                        let refund = await returnPayment(consulation.chargeId)
                        await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -consulation.fee } })
                        if (refund && refund.id) {
                            await TransactionMaster.create({
                                user_id: req.user._id,
                                receiver_id: doctor_id,
                                transaction_id: refund.id,
                                price: consulation.fee,
                                msg: "Refund Payment"
                            })
                        }
                    }
                    await MessageMaster.create({
                        sender_id: doctor_id == consulation.user ? consulation.doctor : consulation.user,
                        receiver_id: doctor_id,
                        msg: consulation._id,
                        msg_type: "Consultation Rejected"
                    });
                    await ConsultationMaster.updateOne({ _id: consulation._id }, { status: "reject" })
                });
                deferred.resolve('doctor_delete_success')
            }
        }
        else {
            deferred.reject("invalid_doctor_id")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.hideUnHideDoctor = async (req, res) => {
    let deferred = Q.defer()
    let { doctor_id, isHide } = req.body;
    try {
        let userResponseObj = await UserMaster.findOneAndUpdate({ _id: doctor_id, deleted_at: null }, { is_hide: isHide }, { new: true }).setOptions({ ignorePreHooks: true });
        if (userResponseObj) {

            if (userResponseObj.user_type == "pet_parent") {
                if (isHide == true) {
                    deferred.resolve("user_hide_successfully")
                } else {
                    deferred.resolve("user_un_hide_successfully")
                }
            } else {
                if (isHide == true) {
                    deferred.resolve("hide_successfully")
                } else {
                    deferred.resolve("un_hide_successfully")
                }
            }
        } else {
            deferred.reject("invalid_doctor_id")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.suspendDoctor = async (req, res) => {
    let deferred = Q.defer()
    let { doctor_id, isSuspend } = req.body;
    try {
        let userResponseObj = await UserMaster.findOneAndUpdate({ _id: doctor_id, deleted_at: null }, { is_suspended: isSuspend }, { new: true }).setOptions({ ignorePreHooks: true });
        if (userResponseObj) {

            if (userResponseObj.user_type == "pet_parent") {
                if (isSuspend == true) {
                    let totalAppo = await ConsultationMaster.find({ user: doctor_id, status: { $in: ["pending", "accept"] }, payment: "success" })
                    totalAppo.map(async (consulation) => {
                        if (consulation.chargeId) {
                            let refund = await returnPayment(consulation.chargeId)
                            await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -consulation.fee } })
                            if (refund && refund.id) {
                                await TransactionMaster.create({
                                    user_id: req.user._id,
                                    receiver_id: doctor_id,
                                    transaction_id: refund.id,
                                    price: consulation.fee,
                                    msg: "Refund Payment"
                                })
                            }
                        }
                        await MessageMaster.create({
                            sender_id: doctor_id == consulation.user ? consulation.doctor : consulation.user,
                            receiver_id: doctor_id,
                            msg: consulation._id,
                            msg_type: "Consultation Rejected"
                        });
                        await ConsultationMaster.updateOne({ _id: consulation._id }, { status: "reject" })
                    });

                    deferred.resolve("user_suspend_successfully")
                } else {

                    deferred.resolve("user_un_suspend_successfully")
                }
            } else {
                if (isSuspend == true) {
                    let totalAppo = await ConsultationMaster.find({ doctor: doctor_id, status: { $in: ["pending", "accept"] }, payment: "success" })

                    totalAppo.map(async (consulation) => {

                        if (consulation.chargeId) {
                            let refund = await returnPayment(consulation.chargeId)
                            await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -consulation.fee } })
                            if (refund && refund.id) {
                                await TransactionMaster.create({
                                    user_id: req.user._id,
                                    receiver_id: doctor_id,
                                    transaction_id: refund.id,
                                    price: consulation.fee,
                                    msg: "Refund Payment"
                                })
                            }
                        }
                        await MessageMaster.create({
                            sender_id: doctor_id == consulation.user ? consulation.doctor : consulation.user,
                            receiver_id: doctor_id,
                            msg: consulation._id,
                            msg_type: "Consultation Rejected"
                        });
                        await ConsultationMaster.updateOne({ _id: consulation._id }, { status: "reject" })
                    });
                    deferred.resolve("suspend_successfully")
                } else {
                    deferred.resolve("un_suspend_successfully")
                }
            }
        } else {
            deferred.reject("invalid_doctor_id")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.getPetDetail = async (req, res) => {
    let deferred = Q.defer()
    let { userId } = req.body;
    try {
        let petDetail = await PetMaster.find({ user_id: userId }).sort({ created_at: -1 })
        if (petDetail) {
            deferred.resolve(petDetail)
        } else {
            deferred.reject("invalid_user_id")
        }
    } catch (error) {
        deferred.reject(error.message)
    }
    return deferred.promise;
}

adminController.updateOrderStatus = async (req, res) => {
    let deferred = Q.defer()
    let { orderId, orderStatus } = req.body;
    try {
        let orderDetail = await OrderMaster.findByIdAndUpdate({ _id: orderId }, { order_status: orderStatus })
        deferred.resolve(orderDetail)
    } catch (errorCode) {
        deferred.reject(errorCode)
    }
    return deferred.promise;
}

adminController.orderList = async (req, res) => {
    let deferred = Q.defer();
    const { page = 1, limit = 10, search } = req.body;
    try {
        let skip = (page - 1) * limit;

        let orderList = await OrderMaster.aggregate([
            // Match successful payments
            { $match: { payment: "success" } },

            // Lookup user details from user_master
            {
                $lookup: {
                    from: 'user_master',
                    localField: 'user',
                    foreignField: '_id',
                    as: 'user_details'
                }
            },
            { $unwind: '$user_details' }, // Flatten user details
            {
                $addFields: {
                    'user_details.profile': {
                        $concat: [`${global.APP_URL}/public/profile/`, '$user_details.profile']
                    }
                }
            },
            // Lookup address details from address_master
            {
                $lookup: {
                    from: 'address_master',
                    localField: 'address',
                    foreignField: '_id',
                    as: 'address_details'
                }
            },
            { $unwind: '$address_details' }, // Flatten address details

            // Unwind product_arr to work with each product
            { $unwind: '$product_arr' },

            // Lookup product details from order_product_master based on product_id
            {
                $lookup: {
                    from: 'order_product_master',
                    localField: 'product_arr.product_id',
                    foreignField: '_id',
                    as: 'product_details',
                    pipeline: [
                        {
                            $lookup: {
                                from: 'product_master',
                                localField: 'product_id',
                                foreignField: '_id',
                                as: 'product_cat_details',
                                pipeline: [
                                    {
                                        $lookup: {
                                            from: 'category_master',
                                            localField: 'category_id',
                                            foreignField: '_id',
                                            as: 'category_details',
                                        }
                                    },
                                    { $project: { category_name: { $arrayElemAt: ['$category_details.name', 0] } } }
                                ]
                            }
                        },
                    ]
                }
            },
            { $unwind: '$product_details' }, // Flatten product details

            // Add product image URL and other product details
            {
                $addFields: {
                    'product_details.product_image': {
                        $concat: [`${global.APP_URL}/public/product/`, '$product_details.product_image']
                    }
                }
            },

            // Group the results back into the original order structure
            {
                $group: {
                    _id: '$_id',
                    user: { $first: '$user' },
                    address: { $first: '$address' },
                    subtotal: { $first: '$subtotal' },
                    shipping: { $first: '$shipping' },
                    payment_method: { $first: '$payment_method' },
                    order_status: { $first: '$order_status' },
                    payment: { $first: '$payment' },
                    total: { $first: '$total' },
                    created_at: { $first: '$created_at' },
                    updated_at: { $first: '$updated_at' },
                    user_details: { $first: '$user_details' },
                    address_details: { $first: '$address_details' },
                    product_arr: {
                        $push: {
                            product_id: '$product_arr.product_id',
                            quantity: '$product_arr.quantity',
                            price: '$product_arr.price',
                            product_details: '$product_details'
                        }
                    }
                }
            },

            // Optionally search by user email
            ...(search ? [
                {
                    $match: {
                        'user_details.email': { $regex: search, $options: 'i' }
                    }
                }
            ] : []),

            // Sort by order ID
            { $sort: { _id: -1 } },

            // Pagination: count total records and get paginated data
            {
                $facet: {
                    metadata: [{ $count: 'totalRecords' }],
                    data: [
                        { $skip: skip },
                        { $limit: limit }
                    ]
                }
            },
            {
                $project: {
                    totalRecords: { $arrayElemAt: ['$metadata.totalRecords', 0] },
                    data: 1
                }
            }
        ]);



        // console.log(orderList);


        let result = orderList[0];
        deferred.resolve({
            totalRecords: result.totalRecords || 0,
            orderList: result.data
        });
    } catch (error) {
        deferred.reject(error.message);
    }
    return deferred.promise;
};

adminController.getOrderDetail = async (req, res) => {
    let deferred = Q.defer()
    let { orderId } = req.body;
    try {
        let orderDetail = await OrderMaster.findById({ _id: orderId })
        deferred.resolve(orderDetail)
    } catch (errorCode) {
        deferred.reject(errorCode)
    }
    return deferred.promise;
}

adminController.getpayoutList = async (req, res) => {
    let deferred = Q.defer()
    let { page = 1, limit = 10, search } = req.body;
    try {
        let matchQuery = {
            // is_withdraw: 'pending'
        }
        if (search) {
            matchQuery.$or = [
                { 'user_details.fname': { $regex: search, $options: 'i' } },
                { 'user_details.lname': { $regex: search, $options: 'i' } },
            ];
        }

        // let payoutReqList = await PayoutHistoryMaster.find({}).populate('user_id', 'fname lname').sort({ _id: -1 }).skip(skip).limit(limit)
        let payoutReqList = await PayoutHistoryMaster.aggregate([
            {
                $lookup: {
                    from: 'user_master',
                    localField: 'user_id',
                    foreignField: '_id',
                    as: 'user_details',
                    pipeline: [
                        {
                            $project: { fname: 1, lname: 1, profile: { $concat: [global.APP_URL + '/public/profile/', '$profile'] }, wallet: { $round: ['$wallet', 2] } }
                        }
                    ]
                }
            },
            { $unwind: '$user_details' },
            { $match: matchQuery },
            { $sort: { _id: -1 } },
            {
                $project: {
                    user_id: 1,
                    amount: 1,
                    is_withdraw: 1,
                    created_at: 1,
                    user_details: 1
                }
            }
        ])

        let totalCount = payoutReqList.slice((page - 1) * limit, page * limit)

        let result = {
            totalRecords: totalCount.length,
            payoutList: payoutReqList
        }

        deferred.resolve(result)
    } catch (errorCode) {
        deferred.reject(errorCode)
    }
    return deferred.promise;
}

adminController.payoutStatusUpdate = async (req, res) => {
    let deferred = Q.defer()
    let { _id, status } = req.body;
    try {

        let payoutReqUpdate = await PayoutHistoryMaster.findOneAndUpdate({ _id: _id }, { is_withdraw: status }, { new: true })
        if (payoutReqUpdate) {

            let User = await UserMaster.findOne({ _id: payoutReqUpdate.user_id })
            if (User) {
                if (status == "accept") {
                    let payout = await payoutStripe(payoutReqUpdate.amount, User.AccountId, 'withdraw amount')
                    console.log("🚀 ~ adminController.payoutStatusUpdate= ~ payout:", payout)

                    // if(payout&&payout._id){}
                    await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -payoutReqUpdate.amount } }, { new: true })
                    await UserMaster.findOneAndUpdate({ _id: payoutReqUpdate.user_id }, { $inc: { wallet: -payoutReqUpdate.amount } }, { new: true })
                }
                await NotificationMaster.create({
                    sender_id: req.user._id,
                    receiver_id: payoutReqUpdate.user_id,
                    msg: `your withdraw request ${status}ed by admin`,
                    msg_type: "text"
                })
                if (User.device_token && User.device_token != "") {
                    await sendFirebaseNotification(User.device_token, 'VETZY', `your withdraw request ${status}ed by admin`)
                }
            }
        }

        deferred.resolve(payoutReqUpdate)
    } catch (errorCode) {
        deferred.reject(errorCode)
    }
    return deferred.promise;
}

module.exports = adminController