const userController = {};
const Q = require('q');
const UserMaster = require("../models/UserMaster");
const config = require('../config/common.config');
const jwt = require('jsonwebtoken');
const stripe = require('stripe')('sk_test_51Gf0TaDf4p1HjB0rVdcdf2elyG63YZzbKxLQzRCNEdTSVUiK6NMzjOVo9DmPAgfeHwIt8l8wglRuEq01GV62UKxn00hXPkEwl6');
const sendMail = require("../utils/sendMail");
const { contactUsMail, registerMail, forgotPasswordMail } = require("../utils/ContentProvider");
const OTPMaster = require('../models/OTPMaster');
const fs = require('fs');
const publicPath = basedir + "/public/";
const ResponseFormatter = require("../utils/response-formatter");
const sendSMS = require('../utils/sendSMS');

const formatter = new ResponseFormatter();

/** APIs */

userController.payViaStripe = async function (card_number, expmonth, expyear, card_cvv, amount, desc = 'Gawal Booking') {
    try {
        const token = await stripe.tokens.create({
            card: {
                number: card_number,
                exp_month: expmonth,
                exp_year: expyear,
                cvc: card_cvv,
            },
        });

        const charge = await stripe.charges.create({
            amount: amount * 100,
            currency: 'usd',
            source: token.id,
            description: desc,
        });

        return charge;

    } catch (err) {
        return err;
    }
}

userController.refreshToken = async function (req, res) {
    const { user_id } = req.body;
    try {
        let checkUser = await userController.getUserDetail(user_id);
        if (checkUser == null) {
            return res.send(formatter.formatResponse({}, 0, "invalid_user_id", false));
        }

        let token = jwt.sign({ _id: user_id }, config.jwt.secret, { expiresIn: config.jwt.token_expiry });
        return res.send(formatter.formatResponse({ token: token, wallet: checkUser.wallet }, 0, "token_updated_success", true));

    } catch (errorCode) {
        console.error("userController.refreshToken - ", errorCode);
        return res.send(formatter.formatResponse({}, 0, errorCode, false));
    }
}

userController.getUserDetail = async (user_id) => {
    let checkUser = await UserMaster.findOne({ _id: user_id, deleted_at: null }).setOptions({ ignorePreHooks: true }).exec();
    return checkUser;
}

userController.sendOtp = async (req, res) => {
    const { ccode, phone, is_signup, email } = req.body;
    try {
        await OTPMaster.deleteOne({ phone: phone, ccode: ccode });
        
        if (is_signup == "true") {
            let userResponseObj = await UserMaster.findOne({ phone: phone, ccode: ccode, user_type: "pet_parent", deleted_at: null }).setOptions({ ignorePreHooks: true });
            if (userResponseObj) {
                return res.send(formatter.formatResponse({}, 0, "already_register_phone", false))
            }
            let emailExist = await UserMaster.findOne({ email: email, user_type: "pet_parent", deleted_at: null });
            if (emailExist) {
                return res.send(formatter.formatResponse({}, 0, "already_register_email", false))
            }
            let d;
            d = new Date();
            d.setSeconds(d.getSeconds() + 300);
            let otpCode = makeid(4);
            let newOtp = OTPMaster();
            newOtp.ccode = ccode;
            newOtp.phone = phone;
            newOtp.otp = otpCode;
            newOtp.expireAt = d;
            await newOtp.save();

            sendSMS(`your verification code is ${otpCode}`, `+${ccode}${phone}`)
            return res.send(formatter.formatResponse({ otpCode }, 1, "otp_send", true));

        } else {
            let userResponseObj = await UserMaster.findOne({ phone: phone, ccode: ccode, user_type: 'pet_parent', deleted_at: null }).setOptions({ ignorePreHooks: true });
            if (!userResponseObj) {
                return res.send(formatter.formatResponse({}, 0, "not_registered_phone", false))
            }
            // console.log("userResponseObj.is_suspended======"+userResponseObj);

            if (userResponseObj && userResponseObj.is_suspended) {
                return res.send(formatter.formatResponse({}, 10, "user_is_suspend", false));
            }
            let d;
            d = new Date();
            d.setSeconds(d.getSeconds() + 300);
            let otpCode = makeid(4);
            let newOtp = OTPMaster();
            newOtp.ccode = ccode;
            newOtp.phone = phone;
            newOtp.otp = otpCode;
            newOtp.expireAt = d;
            await newOtp.save();

            sendSMS(`your verification code is ${otpCode}`, `+${ccode}${phone}`)
            return res.send(formatter.formatResponse({ otpCode }, 1, "otp_send", true));

        }
    } catch (errorCode) {
        console.error("userController.sendOtp- ", errorCode);
        return res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userController.signUp = async (req, res) => {
    const { email, fname, lname, pet_type, device_type, device_token, phone, ccode, otp } = req.body;
    console.log("🚀 ~ userController.signUp= ~ pet_type:", pet_type)
    try {

        let checkUser = await UserMaster.findOne({ phone, ccode, user_type: "pet_parent" }).exec();
        if (checkUser != null) {
            return res.send(formatter.formatResponse({}, 0, "already_register", false))
        }
        else {
            let chckOtp = await OTPMaster.findOne({ ccode, phone, otp });

            if (!chckOtp) {
                return res.send(formatter.formatResponse({}, 1, "invalid_otp", false))
            }
            await chckOtp.delete()

            let profile = req.file ? (req.file.filename ?? '') : '';
            let user = new UserMaster();
            user.email = email;
            user.phone = phone;
            user.ccode = ccode;
            user.fname = fname;
            user.lname = lname;
            user.pet_type = pet_type;
            user.user_type = "pet_parent";
            user.profile = profile;
            user.device_type = device_type;
            user.device_token = device_token;
            user.save(async function (error, document) {
                if (error) {
                    let errorKeys = Object.keys(error.errors);

                    errorKeys.forEach(key => {
                        console.log(error.errors[key].properties.message);
                        res.send(formatter.formatResponse({}, 0, error.errors[key].properties.message, false))
                    });
                }
                if (document) {
                    // OTP sent to mobile setup
                    let token = jwt.sign({ _id: document._id }, config.jwt.secret, { expiresIn: config.jwt.token_expiry });
                    document._doc.token = token;
                    res.send(formatter.formatResponse(document, 1, "signup_success", true));
                }

            });
        }
    } catch (errorCode) {
        console.error("userController.signUp - ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userController.verifyOTP = async (req, res) => {
    const { phone, ccode, otp, device_token = "", device_type = "" } = req.body;
    try {
        let userResponseObj = await UserMaster.findOne({ phone, ccode, user_type: "pet_parent", deleted_at: null }).setOptions({ ignorePreHooks: true });

        if (userResponseObj) {

            if (userResponseObj && userResponseObj.is_suspended) {
                return res.send(formatter.formatResponse({}, 10, "user_is_suspend", false));
            }

            let chckOtp = await OTPMaster.findOne({ ccode, phone, otp });
            if (chckOtp) {
                await chckOtp.delete()
                let user = await UserMaster.findByIdAndUpdate({ _id: userResponseObj._id }, { device_token: device_token, device_type: device_type });

                if (user != null && user.user_type != null && user.user_type == "veterinarian" && user.is_suspended == true) {
                    console.error("authorize failure - ");
                    var finalRes = formatter.formatResponse({}, 10, config.messages["doctor_is_suspend"], false);
                    return res.send(finalRes);
                }

                let token = jwt.sign({ _id: userResponseObj._id }, config.jwt.secret, { expiresIn: config.jwt.token_expiry });
                userResponseObj._doc.token = token;
                userResponseObj._doc.device_token = device_token;
                userResponseObj._doc.device_type = device_type;
                res.send(formatter.formatResponse(userResponseObj, 1, "otp_verified", true))

            } else {
                return res.send(formatter.formatResponse({}, 0, "invalid_otp", false))
            }
        }
        else {
            res.send(formatter.formatResponse({}, 0, "incorrect_phone", false))
        }

    } catch (errorCode) {
        console.error("userController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userController.updateProfile = async function (req, res) {
    const { user_id } = req.body;
    try {
        let profile = req.file ? (req.file.filename ?? '') : '';
        req.body.profile = profile;
        if (profile == '') {
            delete req.body.profile;
        }
        else {
            let oldPath = publicPath + "profile/" + req.user.profile;
            fs.unlink(oldPath, (err) => {
                if (err) {
                    console.error(err);
                }
            })
        }

        let userResponseObj = await UserMaster.findOneAndUpdate({ _id: user_id }, req.body, { new: true });
        if (userResponseObj) {
            return res.send(formatter.formatResponse(userResponseObj, 1, "profile_update", true));
        }
        else {
            return res.send(formatter.formatResponse({}, 0, "invalid_user_id", false));
        }

    } catch (errorCode) {
        console.error("userController.updateProfile - ", errorCode);
        if (req.file && req.file.profile) {
            fs.unlink(
                publicPath + "profile/" + req.file.profile.filename,
                (err) => {
                    if (err) {
                        console.error(err);
                    }
                    console.log("file removed");
                }
            );
        }

        return res.send(formatter.formatResponse({}, 0, errorCode, false));
    }
}

userController.logOut = async function (req, res) {
    const { user_id } = req.body;
    try {
        let userResponseObj = await UserMaster.findByIdAndUpdate({ _id: user_id }, { device_token: "", device_type: "" }).setOptions({ ignorePreHooks: true });
        if (userResponseObj) {
            return res.send(formatter.formatResponse({}, 1, "logout_success", true));
        }
        else {
            return res.send(formatter.formatResponse({}, 0, "invalid_user_id", false));
        }

    } catch (errorCode) {
        console.error("userController.Logout - ", errorCode);
        return res.send(formatter.formatResponse({}, 0, errorCode, false));
    }
}

userController.deleteAccount = async function (req, res) {
    const { reason } = req.body;
    try {
        let userResponseObj = await UserMaster.findOneAndUpdate({ _id: req.user._id }, { deleted_at: new Date(), reason: reason }, { new: true }).setOptions({ ignorePreHooks: true });
        if (userResponseObj) {
            return res.send(formatter.formatResponse(userResponseObj, 1, "delete_success", true));
        }
        else {
            return res.send(formatter.formatResponse({}, 0, "invalid_user_id", false));
        }
    } catch (errorCode) {
        return res.send(formatter.formatResponse({}, 0, errorCode, false));
    }
}

userController.contactUs = async function (req, res) {
    const { email, name, subject, message } = req.body;
    try {

        let clientEmail = process.env.CLIENT_EMAIL || "mayur.kmphasis@gmail.com";
        let mailContent = await contactUsMail(name, email, subject, message);
        sendMail(clientEmail, "Contact Us email from " + APP_NAME, mailContent);
        return res.send(formatter.formatResponse({}, 1, "contact_us_msg", true));
    } catch (errorCode) {
        console.error("userController.contactUs - ", errorCode);
        return res.send(formatter.formatResponse({}, 1, errorCode, false));
    }
}

userController.getProfile = async function (req, res) {
    try {
        let profile = await UserMaster.findOne({ _id: req.user._id })
        return res.send(formatter.formatResponse(profile, 1, "profile_get_success", true));
    } catch (errorCode) {
        console.error("userController.contactUs - ", errorCode);
        return res.send(formatter.formatResponse({}, 1, errorCode, false));
    }
}


function makeid(length) {
    let result = '';
    let characters = '0123456789';
    let charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() *
            charactersLength));
    }
    return result;
}


module.exports = userController;