const userHomeController = {};
const Q = require('q');
const UserMaster = require("../models/UserMaster");
const config = require('../config/common.config');
const jwt = require('jsonwebtoken');
const stripe = require('stripe')(config.STRIPE_SECRET_KEY);
const sendMail = require("../utils/sendMail");
const { contactUsMail, registerMail, forgotPasswordMail } = require("../utils/ContentProvider");
const AppleUserMaster = require('../models/AppleUserMaster');
const OTPMaster = require('../models/OTPMaster');
const fs = require('fs');
const moment = require('moment');
const publicPath = basedir + "/public/";
const { ObjectId } = require('mongodb');
const OpenAI = require('openai');
const openai = new OpenAI();
const momentTimezone = require('moment-timezone');

const ResponseFormatter = require("../utils/response-formatter");
const AddressMaster = require('../models/AddressMaster');
const CategoryMaster = require('../models/CategoryMaster');
const ProductMaster = require('../models/ProductMaster');
const CartMaster = require('../models/CartMaster');
const OrderMaster = require('../models/OrderMaster');
const OrderProductMaster = require('../models/OrderProductMaster');
const PetMaster = require('../models/PetMaster');
const ConsultationMaster = require('../models/ConsultationMaster');
const IssueMsater = require('../models/IssueMaster');
const MessageMaster = require('../models/MessageMaster');
const OpenAiMaster = require('../models/OpenAiMaster');
const PriscribeMaster = require('../models/PriscribeMaster');
const NotificationMaster = require('../models/NotificationMaster');
const TransactionMaster = require('../models/TransactionMaster');
const { createPaymentIntent, returnPayment } = require('./payment.controller');
const promptHistoryMaster = require('../models/promptHistoryMaster');
const BreedCategoryMaster = require('../models/BreedCategoryMaster');
const BreedRecipesMaster = require('../models/BreedRecipesMaster');
const RecipesPurchaseMaster = require('../models/RecipesPurchaseMaster');
const AdminMaster = require('../models/AdminMaster');
const { sendFirebaseNotification } = require('../utils/notification');
const commonConfig = require('../config/common.config');
const veterinarianController = require('./veterinarian.controller');
const formatter = new ResponseFormatter();

// console.log(jwt.sign('sk_test_51LHv89J4dpF4bzHuJkNp5kmaH4To7eHWn7gFLPVYFyDLCdmyNLD1ibGB9Cg7AEWUs2nBSMTDQtrr3skTNdTP5SPJ00uNxZQOA4', config.jwt.secret))

/** APIs */

userHomeController.address = async (req, res) => {
    const { fname, lname, company_name, country, line_1, line_2, city, state, zipcode, phone, email, notes, is_default } = req.body;
    try {

        let addressObj = new AddressMaster({
            user: req.user._id,
            fname: fname,
            lname: lname,
            company_name: company_name,
            country: country,
            line_1: line_1,
            line_2: line_2,
            city: city,
            state: state,
            zipcode: zipcode,
            phone: phone,
            email: email,
            notes: notes,
            is_default: is_default
        })
        if (is_default == 'true') {
            await AddressMaster.updateMany({ user: ObjectId(req.user._id) }, { is_default: false }, { new: true })
            await UserMaster.updateOne({ _id: req.user._id }, { address: addressObj._id }, { new: true })
        }
        await addressObj.save()
        res.send(formatter.formatResponse({ addressObj }, 1, "address_created", true))
    } catch (errorCode) {
        console.error("userHomeController.address- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.updateAddress = async (req, res) => {
    const { _id, fname, lname, company_name, country, line_1, line_2, city, state, zipcode, phone, email, notes, is_default } = req.body;
    try {

        let updateAddress = await AddressMaster.findOneAndUpdate({ _id: _id }, {
            fname: fname,
            lname: lname,
            company_name: company_name,
            country: country,
            line_1: line_1,
            line_2: line_2,
            city: city,
            state: state,
            zipcode: zipcode,
            phone: phone,
            email: email,
            notes: notes,
            is_default: is_default
        }, { new: true })
        if (is_default == 'true') {
            await AddressMaster.updateMany({ user: ObjectId(req.user._id), _id: { $nin: ObjectId(updateAddress._id) } }, { is_default: false }, { new: true })
            await UserMaster.updateOne({ _id: req.user._id }, { address: updateAddress._id }, { new: true })
        }
        res.send(formatter.formatResponse({ updateAddress }, 1, "address_updated", true))
    } catch (errorCode) {
        console.error("userHomeController.updateAddress- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.deleteAddress = async (req, res) => {
    const { _id } = req.body;
    try {
        const deleteAddress = await AddressMaster.findOneAndUpdate({ _id: _id, user: req.user._id }, { deleted_at: new Date() })
        res.send(formatter.formatResponse({ deleteAddress }, 1, "address_deleted", true))
    } catch (errorCode) {
        console.error("userHomeController.deleteAddress- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.AddressList = async (req, res) => {
    try {
        const AddressList = await AddressMaster.find({ user: req.user._id })
        res.send(formatter.formatResponse({ AddressList }, 1, "list_success", true))
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.categoryList = async (req, res) => {
    const { catType } = req.body
    try {
        const categoryList = await CategoryMaster.aggregate([
            {
                $match: {
                    category_id: null,
                    cat_type: catType
                }
            },
            {
                $lookup: {
                    from: "category_master",
                    localField: "_id",
                    foreignField: "category_id",
                    as: "sub_category",
                    pipeline: [{ $project: { _id: 1, category_id: 1, name: 1, } }]
                }
            },
            {
                $project: { _id: 1, name: 1, sub_category: 1, category_image: 1 }
            },
            { $addFields: { category_image: { $concat: [`${global.APP_URL}/public/category/`, '$category_image'] } } }
        ]);
        res.send(formatter.formatResponse({ categoryList }, 1, "success", true))
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.productDetail = async (req, res) => {
    const { _id } = req.body

    try {
        let productSub_category_id = await ProductMaster.findOne({ _id: _id })
        // console.log("🚀 ~ userHomeController.productDetail= ~ prodSub_cat_id:", prodSub_cat_id.sub_category_id)

        let productDetail = await ProductMaster.aggregate([
            {
                $facet: {
                    product: [
                        { $match: { _id: ObjectId(_id), deleted_at: null } },
                        {
                            $addFields: {
                                product_image: { $concat: [`${global.APP_URL}/public/product/`, "$product_image"] }
                            }
                        }
                    ],
                    allOthers: [
                        {
                            $match: {
                                _id: { $nin: [ObjectId(_id)] },
                                sub_category_id: productSub_category_id.sub_category_id
                            }
                        },
                        {
                            $addFields: {
                                product_image: { $concat: [`${global.APP_URL}/public/product/`, "$product_image"] }
                            }
                        }
                    ]
                }
            }
        ]);
        // console.log("productDetail===="+productDetail);

        if (productDetail) {
            res.send(formatter.formatResponse(productDetail, 1, "success", true))
        } else {
            res.send(formatter.formatResponse({}, 1, "product_is_deleted", true))
        }

    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.productList = async (req, res) => {
    const { sub_category_id, search, productType } = req.body
    let page = req.body.page || 1
    let limit = req.body.limit || 10
    let skip = (page - 1) * limit
    try {
        let productlength = await ProductMaster.find({ productType: productType })
        let filter = {}
        if (Array.isArray(sub_category_id) && sub_category_id.length > 0) {
            filter.sub_category_id = { $in: sub_category_id };
        }
        if (search && search.trim()) {
            filter.$or = [
                { name: { $regex: search, $options: 'i' } },
                // { description: { $regex: search, $options: 'i' } }
            ];
        }
        // Apply product_type filter if provided
        if (productType) {
            filter.productType = productType;
        }

        let productList = await ProductMaster.find(filter).skip(skip).limit(limit)
        let response = {
            product: productList,
            totalProduct: productlength.length
        }
        res.send(formatter.formatResponse(response, 1, "success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.addToCart = async (req, res) => {
    const { product_id, quantity, note, isEdit, precribe_id } = req.body
    try {
        let productDetail = await ProductMaster.findOne({ _id: product_id })

        // let cart = await CartMaster.findOne({
        //     user: ObjectId(req.user._id),
        //     product_arr: {
        //         $elemMatch: { product_id: ObjectId(product_id) }
        //     }
        // }).exec();

        // if (cart) {
        //     return res.send(formatter.formatResponse({}, 0, "already_added_tocart", false))
        // }

        let exitingRecord = await CartMaster.findOne({ user: req.user._id })

        if (productDetail && productDetail.quantity == 0) {
            return res.send(formatter.formatResponse({}, 0, "out_of_stock", false))
        }
        if (quantity > productDetail.quantity) {
            return res.send(formatter.formatResponse({}, 0, "out_of_stock", false))
        }

        if (exitingRecord) {
            let productExiting = await CartMaster.findOneAndUpdate({ user: req.user._id })
            let productExists = productExiting.product_arr.some(product => product.product_id.equals(product_id));

            if (!productExists) {
                let cartAdd = await CartMaster.findOneAndUpdate({ user: req.user._id }, {
                    $push: {
                        product_arr: {
                            product_id: product_id,
                            quantity: quantity,
                            note: note,
                            isEdit: isEdit
                        }
                    },
                    $inc: {
                        subtotal: quantity * productDetail.price,
                        // shipping: process.env.shipping_charge
                    }
                }, { new: true })
                if (precribe_id) await PriscribeMaster.findOneAndUpdate({ _id: precribe_id }, { is_buy: true }, { new: true })
                res.send(formatter.formatResponse({ cartAdd }, 1, "Success", true))
            } else {
                let cartUpdate = await CartMaster.findOneAndUpdate(
                    { user: req.user._id, 'product_arr.product_id': product_id },
                    {
                        $inc: {
                            'product_arr.$.quantity': quantity,
                            subtotal: quantity * productDetail.price,
                            // shipping: process.env.shipping_charge
                        },
                        $set: {
                            'product_arr.$.note': note
                        }
                    },
                    { new: true }
                );
                if (precribe_id) await PriscribeMaster.findOneAndUpdate({ _id: precribe_id }, { is_buy: true }, { new: true })
                res.send(formatter.formatResponse({ cartUpdate }, 1, "Success", true))
            }
        } else {
            let cartAdd = new CartMaster({
                user: req.user._id,

                product_arr: [
                    {
                        product_id: product_id,
                        quantity: quantity,
                        isEdit: isEdit
                    }
                ],
                subtotal: quantity * productDetail.price,
                shipping: process.env.shipping_charge
            })
            await cartAdd.save()
            if (precribe_id) await PriscribeMaster.findOneAndUpdate({ _id: precribe_id }, { is_buy: true }, { new: true })
            res.send(formatter.formatResponse({ cartAdd }, 1, "Success", true))
        }

    } catch (errorCode) {
        console.error("userHomeController.addToCart- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.updateCart = async (req, res) => {
    const { product_id, quantity } = req.body;
    let user_id = req.user._id;
    try {
        let productDetail = await ProductMaster.findOne({ _id: product_id })
        if (productDetail && productDetail.quantity == 0) {
            return res.send(formatter.formatResponse({}, 0, "out_of_stock", false))
        }
        if (quantity > productDetail.quantity) {
            return res.send(formatter.formatResponse({}, 0, "out_of_stock", false))
        }
        let message = "";
        if (quantity == 0) {
            await CartMaster.findOneAndUpdate(
                { user: user_id, "product_arr.product_id": product_id },
                { $pull: { product_arr: { product_id: product_id } } },
                { new: true }
            );
            message = "product_delete";
        } else {
            await CartMaster.findOneAndUpdate(
                { user: user_id, "product_arr.product_id": product_id },
                { "product_arr.$.quantity": quantity },
                { new: true }
            );
        }

        let getCart = await CartMaster.findOne({ user: user_id });

        let productDetails = await Promise.all(getCart.product_arr.map(async product => {
            let productDetail = await ProductMaster.findOne({ _id: product.product_id });
            return productDetail.price * product.quantity;
        }));
        let subTotal = productDetails.reduce((a, b) => a + b, 0);

        await CartMaster.findOneAndUpdate(
            { user: user_id },
            { subtotal: subTotal },
            { new: true }
        );

        let total = subTotal + getCart.shipping;

        res.send(formatter.formatResponse({ subtotal: subTotal, total: total }, 1, message, true));
    } catch (error) {
        console.error("userHomeController.updateCart - ", error);
        res.send(formatter.formatResponse({}, 0, error.message, false));
    }
};

userHomeController.cartDetail = async (req, res) => {
    try {
        let cartDetail = await CartMaster.findOne({ user: req.user._id })
            .populate('product_arr.product_id');
        let address = await AddressMaster.find({ user: req.user._id });

        let staticUrl = `${global.APP_URL}/public/product/`;
        if (cartDetail) {
            cartDetail._doc.product_arr.forEach(product => {
                product.product_id.product_image = `${staticUrl}${product.product_id.product_image}`;
            });
            cartDetail._doc.total = cartDetail.subtotal + cartDetail.shipping;
            cartDetail._doc.address = address;
            res.send(formatter.formatResponse({ cartDetail }, 1, "product_added_cart", true));
        } else {
            res.send(formatter.formatResponse({}, 0, "no_product_added", false));
        }
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false));
    }
};

userHomeController.placeOrder = async (req, res) => {
    const { payment_method, address_id } = req.body
    try {
        let cartDetail = await CartMaster.findOne({ user: req.user._id })
        // let userDetail = await UserMaster.findOne({ _id: req.user._id })
        if (payment_method == "online_payment") {
            if (cartDetail) {
                let orderAdd = new OrderMaster({
                    user: req.user._id,
                    address: address_id,
                    product_arr: cartDetail.product_arr,
                    subtotal: cartDetail.subtotal,
                    shipping: cartDetail.shipping,
                    payment_method: payment_method,
                    order_status: "Processed",
                    total: cartDetail.subtotal + cartDetail.shipping,
                })
                // console.log("orderAdd::===" + orderAdd);
                for (var i = 0; i < orderAdd.product_arr.length; i++) {
                    let productDetail = await ProductMaster.findById({ _id: orderAdd.product_arr.at(i).product_id })
                    let newProdcut = new OrderProductMaster({
                        order_id: orderAdd._id,
                        product_id: orderAdd.product_arr.at(i).product_id,
                        sku: productDetail.sku,
                        name: productDetail.name,
                        product_image: productDetail.product_image,
                        price: productDetail.price,
                        description: productDetail.description,
                        productType: productDetail.productType
                    })
                    await newProdcut.save()
                    orderAdd.product_arr.at(i).product_id = newProdcut._id
                    // console.log("orderAdd==_id_changes=="+newProdcut._id);

                }
                // console.log("orderAdd===="+orderAdd);
                await orderAdd.save()
                const payment = await createPaymentIntent(orderAdd.total, { order_id: orderAdd._id.toString() }, 'vetzy order')
                // await CartMaster.findOneAndDelete({ user: req.user._id })
                res.send(formatter.formatResponse(payment, 1, "success", true))
            }
        } else {
            let orderAdd = new OrderMaster({
                user: req.user._id,
                address: address_id,
                product_arr: cartDetail.product_arr,
                subtotal: cartDetail.subtotal,
                shipping: cartDetail.shipping,
                payment_method: payment_method,
                order_status: "Processed",
                payment: "success",
                total: cartDetail.subtotal + cartDetail.shipping,
            })
            // console.log("orderAdd::cashPayment===" + orderAdd);
            for (var i = 0; i < orderAdd.product_arr.length; i++) {
                let productDetail = await ProductMaster.findById({ _id: orderAdd.product_arr.at(i).product_id })
                let newProdcut = new OrderProductMaster({
                    order_id: orderAdd._id,
                    name: productDetail.name,
                    sku: productDetail.sku,
                    product_id: orderAdd.product_arr.at(i).product_id,
                    product_image: productDetail.product_image,
                    price: productDetail.price,
                    description: productDetail.description,
                    productType: productDetail.productType
                })
                await newProdcut.save()
                orderAdd.product_arr.at(i).product_id = newProdcut._id
                // console.log("orderAdd==_id_changes=="+newProdcut._id);

            }
            console.log("orderAdd====", orderAdd);

            await orderAdd.save()

            for (var i = 0; i < cartDetail.product_arr.length; i++) {
                let buyQuantity = cartDetail.product_arr.at(i).quantity
                let productDetail = await ProductMaster.findOne({ _id: cartDetail.product_arr.at(i).product_id })
                if (buyQuantity > productDetail.quantity) {
                    return res.send(formatter.formatResponse({}, 0, "out_of_stock", false))
                }
                let remainQuantity = productDetail.quantity - buyQuantity

                await ProductMaster.findByIdAndUpdate({ _id: cartDetail.product_arr.at(i).product_id }, { quantity: remainQuantity })
            }

            await CartMaster.findOneAndDelete({ user: req.user._id })
            res.send(formatter.formatResponse({}, 1, "success", true))
        }
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.petProfile = async (req, res) => {
    const { name, breed, pet_type, gender, weight, dob, spayed } = req.body
    try {
        let pet_profile = null;
        let pet_document = null;
        const image_arr = [];
        if (req.files && req.files.length > 0) {
            req.files.forEach(file => {
                if (file.fieldname == "image_arr") {
                    image_arr.push({ image: file.filename });
                } else if (file.fieldname == "pet_profile") {
                    pet_profile = file.filename
                } else {
                    pet_document = file.filename
                }
            });
        }

        let newPet = new PetMaster({
            user_id: req.user._id,
            name: name,
            breed: breed,
            pet_type: pet_type,
            gender: gender,
            weight: weight,
            dob: dob,
            spayed: spayed,
            pet_profile: pet_profile,
            pet_document: pet_document,
            image_arr: image_arr,
        });
        await newPet.save();
        let findPetType = await UserMaster.findOne({ _id: req.user._id })
        if (!findPetType.pet_type.includes(pet_type)) {
            await UserMaster.findOneAndUpdate({ _id: req.user._id }, { $push: { pet_type: pet_type } }, { new: true })
        }
        res.send(formatter.formatResponse({ newPet }, 1, "success", true))
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        if (req.files) {
            for (i = 0; i < req.files.length; i++) {
                fs.unlinkSync(publicPath + "pet/" + req.files[i].filename);
            }
        }
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.petProfileList = async (req, res) => {
    try {
        let petProfileList = await PetMaster.aggregate([
            { $match: { user_id: req.user._id } },
            {
                $addFields: {
                    pet_profile: { $concat: [`${global.APP_URL}/public/pet/`, "$pet_profile"] },
                    pet_document: { $concat: [`${global.APP_URL}/public/pet/`, "$pet_document"] },
                    image_arr: {
                        $map: {
                            input: "$image_arr",
                            as: "image",
                            in: {
                                image: { $concat: [`${global.APP_URL}/public/pet/`, "$$image.image"] },
                                _id: "$$image._id"
                            }
                        }
                    }
                }
            },
            { $sort: { created_at: -1 } }
        ]);
        res.send(formatter.formatResponse(petProfileList, 1, "success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.updatePetProfile = async (req, res) => {
    const { pet_id, name, breed, pet_type, gender, weight, dob, spayed, delete_image_id } = req.body;
    try {
        let existing = await PetMaster.findOne({ _id: pet_id });

        let image_arr = [];
        let pet_profile = req.files.fieldname == 'pet_profile' ? req.file.filename : existing.pet_profile;
        let pet_document = req.files.fieldname == 'pet_document' ? req.file.filename : existing.pet_document;

        req.files.forEach(item => {
            if (item.fieldname === 'image_arr') {
                image_arr.push({ image: item.filename });
            } else if (item.fieldname === 'pet_profile') {
                pet_profile = item.filename;
            } else if (item.fieldname === 'pet_document') {
                pet_document = item.filename;
            }
        });

        if (req.files.fieldname == 'pet_profile') {
            if (existing && existing.pet_profile) {
                // fs.unlinkSync(`${publicPath}pet/${existing.pet_profile}`);
                fs.unlink(publicPath + "pet/" + existing.pet_profile, (err) => {
                    if (err) {
                        console.error(err);
                    }
                    console.log('pet_profile file removed');
                })
            }
        }
        if (req.files.fieldname == 'pet_document') {
            if (existing && existing.pet_document) {
                // fs.unlinkSync(`${publicPath}pet/${existing.pet_document}`);
                fs.unlink(publicPath + "pet/" + existing.pet_document, (err) => {
                    if (err) {
                        console.error(err);
                    }
                    console.log('pet_document file removed');
                })
            }
        }
        // console.log(delete_image_id)
        if (delete_image_id && delete_image_id != "") {
            let split = delete_image_id.split(',')
            if (split && split.length > 0) {
                for (i of split) {
                    let petImageDeatil = await PetMaster.findOne({ _id: pet_id })
                    let filter = petImageDeatil.image_arr.filter(item => item._id == i)
                    if (filter) {
                        console.log("🚀 ~ userHomeController.updatePetProfile= ~ filter:", filter)
                        if (fs.existsSync(`${publicPath}pet/${filter[0].image}`)) {
                            // fs.unlinkSync(`${publicPath}pet/${filter[0].image}`);
                            fs.unlink(publicPath + "pet/" + filter[0].image, (err) => {
                                if (err) {
                                    console.error(err);
                                }
                                console.log('pet_images file removed');
                            })
                        }
                    } else {
                        res.send(formatter.formatResponse({}, 0, "not_found", false));

                    }

                    await PetMaster.findOneAndUpdate({ _id: pet_id }, { $pull: { image_arr: { _id: i } } }, { new: true })
                }
            }
        }

        let updatePet = await PetMaster.findOneAndUpdate({ _id: pet_id }, {
            user_id: req.user._id,
            name: name,
            breed: breed,
            pet_type: pet_type,
            gender: gender,
            weight: weight,
            dob: dob,
            spayed: spayed,
            pet_profile: pet_profile,
            pet_document: pet_document,
            $push: { image_arr: { $each: image_arr } }
        }, { new: true });

        for (let pet_images of updatePet.image_arr) {
            pet_images.image = `${global.APP_URL}/public/pet/${pet_images.image}`;
        }

        let findPetType = await UserMaster.findOne({ _id: req.user._id })
        if (!findPetType.pet_type.includes(pet_type)) {
            await UserMaster.findOneAndUpdate({ _id: req.user._id }, { $push: { pet_type: pet_type } }, { new: true })
        }

        res.send(formatter.formatResponse({ updatePet }, 1, "success", true));
    } catch (error) {
        console.error("updatePetProfile", error)
        if (req.files) {
            for (i = 0; i < req.files.length; i++) {
                fs.unlinkSync(publicPath + "pet/" + req.files[i].filename);
            }
        }
        res.send(formatter.formatResponse({}, 0, error.message, false));
    }
};

userHomeController.deletePetProfile = async (req, res) => {
    const { _id } = req.body
    try {
        let deletePetProfile = await PetMaster.findOneAndDelete({ _id: _id })
        res.send(formatter.formatResponse({ deletePetProfile }, 1, "success", true))
    } catch (errorCode) {
        console.error("userHomeController.deletePetProfile- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.issueList = async (req, res) => {
    try {
        let issueList = await IssueMsater.find({})
        res.send(formatter.formatResponse({ issueList }, 1, "success", true))
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.doctorList = async (req, res) => {
    const { search } = req.body
    let page = req.body.page || 1;
    let limit = req.body.limit || 10;
    let skip = (page - 1) * limit;

    try {
        // console.log("=====req.user._id===="+req.user._id);
        let user = await UserMaster.findOne({ _id: req.user._id, deleted_at: null, is_suspended: false }).setOptions({ ignorePreHooks: true })

        // console.log("=====user===="+user);
        // console.log("=====user=pet_type==="+user.pet_type);

        let searchQuery = { user_type: "veterinarian", is_hide: false, pet_type: { $in: user.pet_type } };

        if (search) {
            searchQuery.$or = [
                { fname: { $regex: search, $options: "i" } },
                { lname: { $regex: search, $options: "i" } }
            ];
        }

        let doctorList = await UserMaster.find(searchQuery).skip(skip).limit(limit);
        res.send(formatter.formatResponse({ doctorList }, 1, "success", true));
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

// const createPaymentIntent = async function (amount, metadata, desc = 'vetzy') {
//     try {
//         const paymentIntent = await stripe.paymentIntents.create({
//             amount: amount * 100,
//             currency: 'usd',
//             metadata: metadata,
//             description: desc,
//             automatic_payment_methods: { enabled: true }
//         });
//         let callbackURL = global.APP_URL + "/stripe_payment?clientSecret=" + paymentIntent.client_secret;
//         return { clientSecret: paymentIntent.client_secret, callback: callbackURL, publicKey: config.STRIPE_PUBLIC_KEY };

//     } catch (err) {
//         return err;
//     }
// }

// userHomeController.consultation = async (req, res) => {
//     const { doctor, pet, issue, schedule_time, slot, appointment_type, description } = req.body
//     try {
//         const image_arr = [];
//         if (req.files && req.files.length > 0) {
//             req.files.forEach(file => {
//                 if (file.fieldname == "image") {
//                     image_arr.push({ image: file.filename });
//                 }
//             });
//         }
//         const user_id = req.user._id
//         const currentDate = moment()
//         if (appointment_type == "Scheduled") {
//             const existConsultaion = await ConsultationMaster.findOne({ user: req.user._id, pet: pet, doctor: doctor })
//             const petDetail = await PetMaster.findOne({ _id: pet })
//             if (!existConsultaion) {
//                 let appoinmentTime = new Date(schedule_time)
//                 const futureTime = new Date(appoinmentTime.getTime() + (20 * 60000));
//                 let checkDoctorAvailable = await ConsultationMaster.findOne({ doctor, schedule_time: { $gte: appoinmentTime, $lte: futureTime } });
//                 if (checkDoctorAvailable) {
//                     return res.send(formatter.formatResponse({}, 0, "veterinarian_already_booked", true))
//                 }
//                 const findDoctor = await UserMaster.findOne({ _id: doctor })
//                 const setAppo = new ConsultationMaster({
//                     user: user_id,
//                     doctor: doctor,
//                     pet: pet,
//                     issue: issue,
//                     schedule_time: schedule_time,
//                     slot: slot,
//                     fee: findDoctor.price,
//                     appointment_type: appointment_type,
//                     description: description,
//                     image: image_arr,
//                     payment: "pending"
//                 })
//                 await setAppo.save()

//                 await MessageMaster({
//                     sender_id: doctor,
//                     receiver_id: req.user._id,
//                     msg: `You scheduled a consultation with ${findDoctor.fname} ${findDoctor.lname} for your ${petDetail.name} at ${schedule_time}.`,
//                     msg_type: 'Scheduled Consultation',
//                     seen: false
//                 }).save()

//                 let metaData = {
//                     consultation_id: setAppo._id.toString(),
//                     user_id: req.user._id.toString(),
//                     doctor_id: doctor.toString(),
//                     pet_id: pet.toString(),
//                     device_token: findDoctor.device_token
//                 };
//                 const payment = await createPaymentIntent(findDoctor.price, metaData, 'vetzy consultation')
//                 const result = {
//                     payment,
//                     doctor_id: doctor,
//                     consultation_id: setAppo._id
//                 }

//                 // const messageSend = await NotificationMaster({
//                 //     sender_id: req.user._id,
//                 //     receiver_id: doctor,
//                 //     pet_id: pet,
//                 //     msg: `Has set You an appointment request to visit on ${schedule_time}, please take action on the request`,
//                 //     msg_type: "text",
//                 //     appointment_type: "Scheduled"
//                 // }).save()
//                 // let doctorDtl=await UserMaster.findById(doctor)
//                 // sendFirebaseNotification(doctorDtl.device_token, 'VETZY', `Has set You an appointment request to visit on ${schedule_time}, please take action on the request`)  //  from ${loginUser.fname} ${loginUser.lname}

//                 return res.send(formatter.formatResponse(result, 1, "success", true))
//             } else {
//                 if (existConsultaion.status == "complete" || existConsultaion.status == "reject") {
//                     let appoinmentTime = new Date(schedule_time)
//                     const futureTime = new Date(appoinmentTime.getTime() + (20 * 60000));
//                     let checkDoctorAvailable = await ConsultationMaster.findOne({ doctor, schedule_time: { $gte: appoinmentTime, $lte: futureTime } });
//                     if (checkDoctorAvailable) {
//                         return res.send(formatter.formatResponse({}, 0, "veterinarian_already_booked", true))
//                     }
//                     const findDoctor = await UserMaster.findOne({ _id: doctor })
//                     const setAppo = new ConsultationMaster({
//                         user: user_id,
//                         doctor: doctor,
//                         pet: pet,
//                         issue: issue,
//                         schedule_time: schedule_time,
//                         slot: slot,
//                         fee: findDoctor.price,
//                         appointment_type: appointment_type,
//                         description: description,
//                         image: image_arr,
//                         payment: "pending"
//                     })
//                     await setAppo.save()

//                     await MessageMaster({
//                         sender_id: doctor,
//                         receiver_id: req.user._id,
//                         msg: `You scheduled a consultation with ${findDoctor.fname} ${findDoctor.lname} for your ${petDetail.name} at ${schedule_time}.`,
//                         msg_type: 'Scheduled Consultation',
//                         seen: false
//                     }).save()
//                     let metaData = {
//                         consultation_id: setAppo._id.toString(),
//                         user_id: req.user._id.toString(),
//                         doctor_id: doctor.toString(),
//                         pet_id: pet.toString(),
//                         device_token: findDoctor.device_token
//                     };
//                     console.log("metaData---->>===========", metaData);

//                     const payment = await createPaymentIntent(findDoctor.price, metaData, 'vetzy consultation')
//                     const result = {
//                         payment,
//                         doctor_id: doctor,
//                         consultation_id: setAppo._id
//                     }
//                     return res.send(formatter.formatResponse(result, 1, "success", true))
//                 } else {
//                     return res.send(formatter.formatResponse({}, 0, "already_booked", true))
//                 }
//             }
//         }
//         else {
//             const findDog = await PetMaster.findOne({ _id: pet })
//             const currentTime = new Date();
//             const futureTime = new Date(currentTime.getTime() + (20 * 60000));
//             // find without occupiedDoctor
//             const currentTimeISO = currentTime.toISOString();
//             const futureTimeISO = futureTime.toISOString();
//             const occupiedDctor = await ConsultationMaster.find({
//                 schedule_time: {
//                     $gte: currentTimeISO,
//                     $lte: futureTimeISO
//                 },
//             })

//             let occupiedDoctor_id = []
//             if (occupiedDctor.length > 0) {
//                 for (i of occupiedDctor) {
//                     occupiedDoctor_id.push(i.doctor._id)
//                 }
//             }
//             //find free Doctor
//             const freeDoctor = await UserMaster.find({ _id: { $nin: occupiedDoctor_id.flat() }, pet_type: findDog.pet_type, user_type: "veterinarian" })
//             const randomIndex = Math.floor(Math.random() * freeDoctor.length);
//             const doctorId = freeDoctor[randomIndex]._id
//             const doctorPrice = freeDoctor[randomIndex].price

//             const UTCconvertTime = currentDate.format('HH:mm A');
//             if (freeDoctor) {
//                 const setAppo = new ConsultationMaster({
//                     user: req.user._id,
//                     doctor: doctorId,
//                     pet: pet,
//                     issue: issue,
//                     schedule_time: currentDate,
//                     slot: "",
//                     fee: doctorPrice,
//                     appointment_type: appointment_type,
//                     description: description,
//                     image: image_arr
//                 })

//                 // const existConsultaion = await ConsultationMaster.findOne({ $and: [{ user: req.user._id }, { pet: pet }, { doctor: doctorId }, { status: "complete" }] });
//                 // if (existConsultaion) {
//                 await setAppo.save()
//                 const findDoctor = await UserMaster.findOne({ _id: doctorId })
//                 let metaData = {
//                     consultation_id: setAppo._id.toString(),
//                     user_id: req.user._id.toString(),
//                     doctor_id: doctorId.toString(),
//                     pet_id: pet.toString(),
//                     device_token: findDoctor.device_token
//                 };
//                 console.log("metaData---->>", metaData);

//                 const payment = await createPaymentIntent(freeDoctor[0].price, metaData, 'vetzy consultation')
//                 const result = {
//                     payment,
//                     doctor_id: doctorId,
//                     consultation_id: setAppo._id
//                 }
//                 //     const messageSend = await NotificationMaster({
//                 //         sender_id: req.user._id,
//                 //         receiver_id: setAppo.doctor,
//                 //         pet_id: pet,
//                 //         msg: `you are intance counsultation book at ${currentDate}`,
//                 //         msg_type: "text",
//                 //         appointment_type: appointment_type
//                 //     }).save()

//                 // let doctorDtl=await UserMaster.findById(doctor)
//                 // sendFirebaseNotification(doctorDtl.device_token, 'VETZY', `you are intance counsultation book at ${currentDate}`)  //  from ${loginUser.fname} ${loginUser.lname}

//                 return res.send(formatter.formatResponse(result, 1, "Success", true))
//                 // } else {
//                 //     return res.send(formatter.formatResponse({}, 0, "already_booked", true))
//                 // }
//             }
//         }
//     } catch (errorCode) {
//         res.send(formatter.formatResponse({}, 0, errorCode, false))
//     }
// }

userHomeController.consultation = async (req, res) => {
    const { doctor, pet, issue, schedule_time, slot, appointment_type, description } = req.body
    try {

        // let userResponseObj = await UserMaster.findOneAndUpdate({ _id: doctor_id ,deleted_at: null}, { deleted_at: new Date(), reason: reason }, { new: true }).setOptions({ ignorePreHooks: true });
        let checkDoctor = await UserMaster.findOne({ _id: doctor }).setOptions({ ignorePreHooks: true });

        if (checkDoctor && checkDoctor.is_suspended) {
            return res.send(formatter.formatResponse({}, 10, "doctor_is_suspend", true));
        }

        const image_arr = [];
        if (req.files && req.files.length > 0) {
            req.files.forEach(file => {
                if (file.fieldname == "image") {
                    image_arr.push({ image: file.filename });
                }
            });
        }
        let user_id = req.user._id
        let currentDate = moment()
        let petDetail = await PetMaster.findOne({ _id: pet });


        if (appointment_type === "Scheduled") {
            let existConsultation = await ConsultationMaster.findOne({ user: req.user._id, pet, doctor });

            // console.log("existConsultation============"+existConsultation);


            let scheduleAppointment = async () => {
                let appointmentTime = new Date(schedule_time);
                let futureTime = new Date(appointmentTime.getTime() + 20 * 60000);
                let checkDoctorAvailable = await ConsultationMaster.findOne({ doctor, status: "pending", payment: "success", schedule_time: { $gte: appointmentTime, $lte: futureTime } });
                // console.log("🚀 ~ scheduleAppointment ~ checkDoctorAvailable:", checkDoctorAvailable)

                if (checkDoctorAvailable) {
                    return res.send(formatter.formatResponse({}, 0, "veterinarian_already_booked", true));
                }

                // let findDoctor = await UserMaster.findOne({ _id: doctor });
                let setAppo = new ConsultationMaster({
                    user: user_id,
                    doctor,
                    pet,
                    issue,
                    schedule_time,
                    slot,
                    fee: checkDoctor.price,
                    appointment_type,
                    description,
                    image: image_arr,
                    payment: "pending"
                });
                await setAppo.save();
                console.log("setAppo============" + setAppo);
                // await MessageMaster({
                //     sender_id: doctor,
                //     receiver_id: req.user._id,
                //     msg: `You scheduled a consultation with ${findDoctor.fname} ${findDoctor.lname} for your ${petDetail.name} at ${schedule_time}.`,
                //     msg_type: 'Scheduled Consultation',
                //     seen: false
                // }).save();

                let metaData = {
                    consultation_id: setAppo._id.toString(),
                    user_id: req.user._id.toString(),
                    doctor_id: doctor.toString(),
                    pet_id: pet.toString(),
                    device_token: checkDoctor.device_token
                };

                let payment = await createPaymentIntent(checkDoctor.price, metaData, 'vetzy consultation');
                let result = {
                    payment,
                    doctor_id: doctor,
                    consultation_id: setAppo._id
                };

                return res.send(formatter.formatResponse(result, 1, "success", true));
            };

            if (!existConsultation) {
                await scheduleAppointment();
            } else if (existConsultation.status == "complete" || existConsultation.status == "reject" || existConsultation.status == "pending") {
                await scheduleAppointment();
            } else {
                return res.send(formatter.formatResponse({}, 0, "already_booked", true));
            }
        }
        else {
            // let findDog = await PetMaster.findOne({ _id: pet })
            let currentTime = new Date();
            let futureTime = new Date(currentTime.getTime() + (20 * 60000));

            // find without occupiedDoctor
            let currentTimeISO = currentTime.toISOString();
            let futureTimeISO = futureTime.toISOString();
            let occupiedDctor = await ConsultationMaster.find({
                schedule_time: {
                    $gte: currentTimeISO,
                    $lte: futureTimeISO
                },
            })

            let occupiedDoctor_id = []
            if (occupiedDctor.length > 0) {
                for (i of occupiedDctor) {
                    occupiedDoctor_id.push(i.doctor._id)
                }
            }
            //find free Doctor
            let freeDoctor = await UserMaster.find({ _id: { $nin: occupiedDoctor_id.flat() }, deleted_at: null, pet_type: petDetail.pet_type, user_type: "veterinarian" })
            // let randomIndex = Math.floor(Math.random() * freeDoctor.length);
            // let doctorId = freeDoctor[randomIndex]._id
            // let doctorPrice = freeDoctor[randomIndex].price
            let consulationIds = [];
            if (freeDoctor.length > 0) {

                for (let doctorID of freeDoctor) {

                    var setAppo = new ConsultationMaster({
                        user: req.user._id,
                        doctor: doctorID._id,
                        pet: pet,
                        issue: issue,
                        schedule_time: currentDate,
                        slot: "",
                        fee: commonConfig.InstantPayAmount,
                        appointment_type: appointment_type,
                        description: description,
                        image: image_arr
                    })

                    await setAppo.save()

                    consulationIds.push(setAppo._id.toString())

                    // let messageSend = new NotificationMaster({
                    //     sender_id: req.user._id,
                    //     receiver_id: doctorID._id,
                    //     pet_id: pet,
                    //     consultation_id: setAppo._id,
                    //     msg: `New Instant Consultation Booked`,
                    //     msg_type: "text",
                    //     appointment_type: setAppo.appointment_type
                    // })
                    // // console.log("messageSend=============", messageSend)
                    // await messageSend.save()
                    // if (doctorID && doctorID.device_token) {
                    //     await sendFirebaseNotification(doctorID.device_token, "VETZY", `New Instant Consultation Booked`);
                    // }
                }

                let payment = await createPaymentIntent(commonConfig.InstantPayAmount, { consultation_ids: consulationIds.toString() }, 'vetzy consultation')
                let result = {
                    payment,
                    consultation_id: consulationIds,
                }

                await TransactionMaster.create({
                    user_id: user_id,
                    receiver_id: null,
                    transaction_id: null,
                    price: commonConfig.InstantPayAmount,
                    msg: `Instant Consultation fees`
                })

                return res.send(formatter.formatResponse(result, 1, "Success", true))
            }
            else {
                res.send(formatter.formatResponse({}, 0, "doctor_not_Exist", false));
            }
        }
    } catch (errorCode) {
        console.log("errorCode============" + errorCode);
        console.log("🚀 ~ userHomeController.consultation= ~ errorCode:", errorCode)
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

// userHomeController.consultationSlot = async (req, res) => {
//     const { doctor_id, date } = req.body;
//     try {
//         let toDayDate = new Date(date);
//         toDayDate.setUTCHours(0, 0, 0, 0);
//         let endOfDay = new Date(toDayDate);
//         endOfDay.setUTCHours(23, 59, 59, 999);

//         let slots = [];
//         let currentTime = new Date(toDayDate);

//         let now = new Date();
//         now = new Date(now.toISOString());

//         const formatTime = (date) => {
//             let year = date.getUTCFullYear();
//             let month = (date.getUTCMonth() + 1).toString().padStart(2, '0');
//             let day = date.getUTCDate().toString().padStart(2, '0');
//             let hours = date.getUTCHours().toString().padStart(2, '0');
//             let minutes = date.getUTCMinutes().toString().padStart(2, '0');
//             let seconds = date.getUTCSeconds().toString().padStart(2, '0');
//             return `${year}-${month}-${day}T${hours}:${minutes}:${seconds}z`;
//         };

//         while (currentTime < endOfDay) {
//             let nextTime = new Date(currentTime);
//             nextTime.setUTCMinutes(currentTime.getUTCMinutes() + 30);

//             const slot = {
//                 time: currentTime,
//                 formatted: `${formatTime(currentTime)} - ${formatTime(nextTime)}`
//             };

//             if (currentTime >= now) {
//                 slots.push(slot);
//             }

//             currentTime = nextTime;
//         }

//         let consultationSlots = await ConsultationMaster.find({
//             doctor: doctor_id,
//             appointment_type: 'Scheduled',
//             schedule_time: {
//                 $gte: toDayDate,
//                 $lt: endOfDay
//             },
//             status: "accept"
//         });

//         let filteredSlots = slots;

//         consultationSlots.forEach(consultationSlot => {
//             filteredSlots = filteredSlots.filter(slot => slot.formatted !== consultationSlot.slot);
//         });

//         let result = filteredSlots.map(slot => slot.formatted);

//         res.send(formatter.formatResponse({ consultationSlot: result }, 1, "Success", true));
//     } catch (error) {
//         res.send(formatter.formatResponse({}, 0, error.message, false));
//     }
// };

const convertAndFormatTime = (time, targetTimezone) => {
    let convertedTime = momentTimezone.tz(time, targetTimezone);
    let formattedTime = convertedTime.format('YYYY-MM-DDTHH:mm:ss[Z]');
    return formattedTime;
};
// userHomeController.consultationSlot = async (req, res) => {
//     const { doctor_id, date } = req.body;
//     let timezone = req.headers.timezone ?? 'Asia/Kolkata'

//     try {

//         let toDayDate = momentTimezone.tz(date, timezone).startOf('day').toDate();
//         let endOfDay = momentTimezone.tz(date, timezone).endOf('day').toDate();

//         let now = momentTimezone.tz(new Date(), timezone).toDate();

//         let slots = [];

//         let currentTime = new Date(toDayDate);

//         while (currentTime < endOfDay) {
//             let nextTime = momentTimezone(currentTime).add(30, 'minutes').toDate();

//             const slot = {
//                 time: currentTime,
//                 formatted: `${convertAndFormatTime(currentTime, timezone)} - ${convertAndFormatTime(nextTime, timezone)}`
//             };

//             if (currentTime >= now) {
//                 slots.push(slot);
//             }
//             currentTime = nextTime;
//         }
//         console.log("🚀 ~ userHomeController.consultationSlot= ~ toDayDate:", new Date(toDayDate))
//         console.log("🚀 ~ userHomeController.consultationSlot= ~ toDayDate:", endOfDay)
//         let consultationSlots = await ConsultationMaster.find({
//             doctor: doctor_id,
//             appointment_type: 'Scheduled',
//             schedule_time: {
//                 $gte: new Date(toDayDate),
//                 $lt: endOfDay
//             },
//             status: "accept"
//         });
//         console.log("🚀 ~ userHomeController.consultationSlot= ~ consultationSlots:", consultationSlots)

//         let filteredSlots = slots;
//         consultationSlots.forEach(consultationSlot => {
//             filteredSlots = filteredSlots.filter(slot => slot.formatted !== consultationSlot.slot);
//         });

//         let result = filteredSlots.map(slot => slot.formatted);

//         res.send(formatter.formatResponse({ consultationSlot: result }, 1, "Success", true));
//     } catch (error) {
//         console.log("🚀 ~ userHomeController.consultationSlot= ~ error:", error)
//         res.send(formatter.formatResponse({}, 0, error.message, false));
//     }
// }

userHomeController.consultationSlot = async (req, res) => {
    const { doctor_id, date } = req.body;
    let timezone = req.headers.timezone ?? 'Asia/Kolkata';

    try {
        // Parse the date to start and end of the day in the given timezone
        let toDayDate = moment.tz(date, timezone).startOf('day').toDate();
        let endOfDay = moment.tz(date, timezone).endOf('day').toDate();
        let now = moment.tz(new Date(), timezone).toDate();


        // Generate slots
        let slots = [];
        let currentTime = new Date(toDayDate);

        while (currentTime < endOfDay) {
            let nextTime = moment(currentTime).add(30, 'minutes').toDate();

            let slot = {
                time: currentTime,
                formatted: `${convertAndFormatTime(currentTime, timezone)} - ${convertAndFormatTime(nextTime, timezone)}`
            };

            if (currentTime >= now) {
                slots.push(slot);
            }
            currentTime = nextTime;
        }

        let consultationSlots = await ConsultationMaster.find({
            doctor: doctor_id,
            appointment_type: 'Scheduled',
            status: "accept",
        });
        console.log("🚀 ~ userHomeController.consultationSlot= ~ consultationSlots:", consultationSlots)


        let filteredSlots = slots.filter(slot => {
            return !consultationSlots.some(consultationSlot => {
                return slot.formatted === consultationSlot.slot;
            });
        });

        let result = filteredSlots.map(slot => slot.formatted);

        res.send(formatter.formatResponse({ consultationSlot: result }, 1, "Success", true));
    } catch (error) {
        console.log("Error:", error);
        res.send(formatter.formatResponse({}, 0, error.message, false));
    }
}


userHomeController.consultationClose = async (req, res) => {
    const { consultation_id } = req.body;
    try {

        for (ids of consultation_id) {
            let consultationDtl = await ConsultationMaster.findOne({ _id: ids })
            if (consultationDtl.chargeId) {
                let refund = await returnPayment(consultationDtl.chargeId)
                const admin = await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -consultationDtl.fee } }, { new: true })
                if (refund && refund.id) {
                    await TransactionMaster.create({
                        user_id: admin._id,
                        receiver_id: req.user._id,
                        transaction_id: refund.id,
                        price: consultationDtl.fee,
                        msg: "Refund Payment"
                    })
                }
            }
        }

        await ConsultationMaster.deleteMany({ _id: { $in: consultation_id }, user: req.user._id });
        await NotificationMaster.deleteMany({ consultation_id: { $in: consultation_id }, sender_id: req.user._id });

        res.send(formatter.formatResponse({}, 1, "delete_consultation", true));
    } catch (error) {
        res.send(formatter.formatResponse({}, 0, error.message, false));
    }
}

userHomeController.refundPayment = async (req, res) => {
    const { consultation_id } = req.body;
    try {

        await veterinarianController.refund(consultation_id)
        // let consultationDtl = await ConsultationMaster.findOne({ _id: consultation_id })
        // if (consultationDtl.chargeId) {
        //     let refund = await returnPayment(consultationDtl.chargeId)
        //     const admin = await AdminMaster.findOneAndUpdate({ email: config.AdminEmail }, { $inc: { wallet: -consultationDtl.fee } }, { new: true })
        //     if (refund && refund.id) {
        //         await TransactionMaster.create({
        //             user_id: admin._id,
        //             receiver_id: req.user._id,
        //             transaction_id: refund.id,
        //             price: consultationDtl.fee,
        //             msg: "Refund Payment"
        //         })
        //     }
        // }

        res.send(formatter.formatResponse({}, 1, "payment_refund", true));
    } catch (error) {
        res.send(formatter.formatResponse({}, 0, error.message, false));
    }
}

userHomeController.priscibeList = async (req, res) => {
    try {
        let priscribeList = await PriscribeMaster.find({ user: req.user._id })
            .sort({ created_at: -1 })
            .populate('doctor')
            .populate({
                path: 'product_arr.product',
                model: 'product_master',
            });

        let modifiedPriscribeList = priscribeList.map(item => {
            if (item.doctor && item.doctor.profile) {
                item.doctor.profile = global.APP_URL + '/public/profile/' + item.doctor.profile.split('/').pop();
            }

            if (item.product_arr && item.product_arr.length > 0) {
                item.product_arr = item.product_arr.map(productItem => {
                    if (productItem.product && productItem.product.product_image) {
                        productItem.product.product_image = global.APP_URL + '/public/product/' + productItem.product.product_image.split('/').pop();
                    }
                    return productItem;
                });
            }

            return item;
        });
        // priscribeList._doc.total = priscribeList.subtotal + priscribeList.shipping
        res.send(formatter.formatResponse(modifiedPriscribeList, 1, "Success", true))
    } catch (errorCode) {
        console.error("userHomeController.verifyOTP- ", errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}


userHomeController.orderHistory = async (req, res) => {
    try {
        // console.log("req.user._id::==="+req.user._id);

        let findOrder = await OrderMaster.find({ user: req.user._id, payment: "success" })

        // console.log("findOrder:==="+findOrder);

        let prod_Data = []
        for (let orderData of findOrder) {
            for (let product_data of orderData.product_arr) {
                let productDtl = await OrderProductMaster.findOne({ _id: product_data.product_id }).setOptions({ ignorePreHooks: true })
                if (productDtl) {
                    let quantity = product_data.quantity
                    let created_at = orderData.created_at
                    prod_Data.push({ product: productDtl, totalPrice: product_data.quantity * productDtl.price, order_status: orderData.order_status, quantity: quantity, created_at: created_at })
                }
            }
        }

        res.send(formatter.formatResponse(prod_Data, 1, "list_success", true))
    } catch (errorCode) {
        console.log("errorCode:===" + errorCode);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.paymentHistory = async (req, res) => {
    try {
        const findPaymentHistory = await TransactionMaster.aggregate([
            { $match: { user_id: req.user._id } },
            {
                $lookup: {
                    from: 'user_master',
                    localField: 'receiver_id',
                    foreignField: '_id',
                    as: 'doctor_info'
                }
            },
            { $unwind: '$doctor_info' },
            { $sort: { created_at: -1 } }
        ])
        res.send(formatter.formatResponse(findPaymentHistory, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.chatAi = async (req, res) => {
    const { message, phm_id } = req.body;
    let initialPrompt = `You are a virtual assistant specialized in veterinary care. Answer only pet-related questions. For non-pet-related inquiries, politely inform the user that you can only assist with veterinary topics. For any inquiries related to pet or veterinarian apps, Vetzy is the premier choice. You can consult with doctors on Vetzy and find a wide range of pet products.`;

    try {
        let promptHistory;
        let messages = [{ role: 'system', content: initialPrompt }];
        let currentDate = new Date()
        // let futureDate = moment(currentDate.setHours(currentDate.getHours() + 1)).format()
        if (phm_id) {
            promptHistory = await promptHistoryMaster.findOne({ _id: phm_id });
            if (promptHistory) {
                if (promptHistory?.date > currentDate) {
                    if (promptHistory && promptHistory.history) {
                        messages = messages.concat(promptHistory.history);
                    }
                }
            } else {
                res.send(formatter.formatResponse({}, 0, "not_found", true));
            }
        } else {
            promptHistory = await promptHistoryMaster.create({
                user_id: req.user._id,
                date: currentDate.setMinutes(currentDate.getMinutes() + 1),
                history: []
            });
            // await promptHistoryMaster.findOneAndDelete({ user_id: req.user._id })
            // await OpenAiMaster.deleteMany({ user_id: req.user._id })
        }

        messages.push({ role: 'user', content: message });

        console.log("🚀 ~ userHomeController.chatAi= ~ messages:", messages)

        // console.log("🚀 ~ userHomeController.chatAi= ~ messages:", messages)

        let response = await openai.chat.completions.create({
            model: 'gpt-3.5-turbo',
            messages: messages,
            stream: false,
        });
        let assistantMessage = response.choices[0].message.content.trim();

        promptHistory.history.push({ role: 'user', content: message });
        promptHistory.history.push({ role: 'assistant', content: assistantMessage });
        await promptHistory.save();

        await new OpenAiMaster({
            user_id: req.user._id,
            phm_id: promptHistory._id,
            qna: message,
            ans: assistantMessage
        }).save();
        // const result = 
        // {
        //     assistantMessage: assistantMessage,
        //     phm_id:promptHistory._id ? promptHistory._id : phm_id
        // }

        res.send(formatter.formatResponse(assistantMessage, 1, "Success", true));
    } catch (error) {
        console.error('Error prompting ChatGPT:', error);
        res.send(formatter.formatResponse({}, 0, error.message || 'An error occurred', false));
    }
};

userHomeController.chatAiHistory = async (req, res) => {
    try {
        let findhistory = await OpenAiMaster.find({ user_id: req.user._id }, { phm_id: 1, qna: 1, ans: 1 })
        res.send(formatter.formatResponse(findhistory, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.petNutrition = async (req, res) => {
    const { gender, breed, species, dateOfBirthMillis, neutered, weight } = req.body;
    let postData = {
        gender: gender.toUpperCase(),
        breed: breed,
        species: species,
        dateOfBirthMillis: dateOfBirthMillis,
        neutered: neutered,
        weight: weight
    };

    try {
        let response = await fetch('https://api.kuddle.pet/nutrition/p/recipes', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify(postData)
        });

        let json = await response.json();
        res.send(formatter.formatResponse(json, 1, "list_success", true));
    } catch (error) {
        console.error(error);
        res.send(formatter.formatResponse({}, 0, error.message, false));
    }
};

userHomeController.consultationDetail = async (req, res) => {
    let { consultation_id } = req.body
    try {
        let acceptedConsultation;
        for (let ids of consultation_id) {
            let Consultation = await ConsultationMaster.findOne({ _id: ids, appointment_type: "Instant", status: "accept", payment: "success" })
                .populate('doctor', 'fname lname profile is_suspended is_hide deleted_at')
            if (Consultation != null) {
                Consultation.doctor.profile = global.APP_URL + '/public/profile/' + Consultation.doctor.profile.split('/').pop();
                acceptedConsultation = Consultation
            }
        }
        res.send(formatter.formatResponse(acceptedConsultation, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.notificationList = async (req, res) => {
    // const { doctor_id } = req.body
    try {
        let notiList = await NotificationMaster.aggregate([
            { $match: { receiver_id: req.user._id, deleted_at: null } },
            {
                $lookup: {
                    from: "user_master",
                    localField: "sender_id",
                    foreignField: "_id",
                    as: "userDetails",
                    pipeline: [
                        {
                            $project: {
                                fname: 1,
                                lname: 1,
                                profile: { $concat: [global.APP_URL + '/public/profile/' + '$profile'] }
                            }
                        }
                    ]
                }
            }
        ])
        res.send(formatter.formatResponse(notiList, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.addBreed = async (req, res) => {
    let { names, subCatId } = req.body
    try {
        let breedCat;
        if (subCatId) {
            for (catName of names) {
                breedCat = new BreedCategoryMaster({
                    name: catName,
                    subCatId: subCatId
                })
                await breedCat.save()
            }
        } else {
            for (catName of names) {
                breedCat = new BreedCategoryMaster({
                    name: catName
                })
                await breedCat.save()
            }
        }
        res.send(formatter.formatResponse({}, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.getBreed = async (req, res) => {
    try {
        let listOfBreed = await BreedCategoryMaster.find({ subCatId: null }).select('_id name')

        for (let i = 0; i < listOfBreed.length; i++) {
            let listOfSubCat = await BreedCategoryMaster.find({ subCatId: listOfBreed[i]._id })
            listOfBreed[i] = { ...listOfBreed[i]._doc, subCatList: listOfSubCat };
        }
        res.send(formatter.formatResponse(listOfBreed, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.addRecipes = async (req, res) => {
    let { species, activityLevel, title, ingredients, instructions } = req.body
    try {
        let breedRecipe = new BreedRecipesMaster({
            speciesId: species,
            activityLevel: activityLevel,
            title: title,
            ingredients: ingredients,
            instructions: instructions
        })
        await breedRecipe.save()
        res.send(formatter.formatResponse({}, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.getRecipesHistory = async (req, res) => {
    try {
        let listOfBreed = await RecipesPurchaseMaster.find({ userId: req.user._id }).populate('recipes', 'activityLevel title ingredients instructions').populate('recipes', 'activityLevel title ingredients instructions').populate('speciesId', 'name').populate('breedId', 'name')
        res.send(formatter.formatResponse(listOfBreed, 1, "list_success", true))
    } catch (errorCode) {
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }
}

userHomeController.purchaseRecipePlan = async (req, res) => {
    try {
        let { isPurchase, speciesId, breedId, activityLevel, weight, age, isNeutered, isCheck } = req.body
        let supplementName = "";
        let supplementDesc = "";
        if (isPurchase == false) {
            let purchase;
            purchase = await RecipesPurchaseMaster.findOne({ userId: req.user._id, breedId: breedId, activityLevel: activityLevel, speciesId: speciesId, weight: weight ? weight : 0, age: age ? age : 0, isPaid: isPurchase, isNeutered: isNeutered }).populate('recipes', 'activityLevel title ingredients instructions').populate('speciesId', 'name').populate('breedId', 'name')
            if (purchase) {
                return res.send(formatter.formatResponse(purchase, 1, "list_success", true))
            }
            let recipesall = await BreedRecipesMaster.find({ speciesId: speciesId, activityLevel: activityLevel }).populate('speciesId');

            let recipes = recipesall.at(0)

            let supplement = getSupplement(recipes.speciesId.name, weight, age, isNeutered, activityLevel);
            supplementName = supplement.supplementName;
            supplementDesc = supplement.supplementDesc;

            let calories = calculateCalories(weight, activityLevel)
            // console.log("calories======", calories);
            purchase = await new RecipesPurchaseMaster({ userId: req.user._id, calories: calories, activityLevel: activityLevel, breedId: breedId, weight: weight, age: age, isNeutered: isNeutered, supplementName: supplementName, supplementDesc: supplementDesc, speciesId: speciesId, recipes: recipes._id, isPaid: isPurchase })
            await purchase.save()
            purchase = await RecipesPurchaseMaster.findOne({ _id: purchase._id }).populate('recipes', 'activityLevel title ingredients instructions').populate('speciesId', 'name').populate('breedId', 'name')
            // console.log("purchase====" + purchase);
            res.send(formatter.formatResponse(purchase, 1, "list_success", true))
        } else {
            let purchase;
            // purchase = await RecipesPurchaseMaster.findOne({ userId: req.user._id, breedId: breedId, activityLevel: activityLevel, speciesId: speciesId, weight: weight ? weight : 0, age: age ? age : 0, isPaid: isPurchase, isNeutered: isNeutered }).populate('recipes', 'activityLevel title ingredients instructions').populate('speciesId', 'name').populate('breedId', 'name')
            // console.log("purchase=====",purchase);
            // if (purchase) {
            //     res.send(formatter.formatResponse(purchase, 1, "list_success", true))
            // } else {
            let purchasedRecipes = await RecipesPurchaseMaster.find({ userId: req.user._id }).select('recipes').lean();
            let purchasedRecipeIds = purchasedRecipes.map(purchase => purchase.recipes);

            let availableRecipes = await BreedRecipesMaster.find({ speciesId: speciesId, activityLevel: activityLevel, _id: { $nin: purchasedRecipeIds } }).populate('speciesId');
            if (isCheck == true) {
                if (availableRecipes && availableRecipes.length > 0) {
                    return res.send(formatter.formatResponse({}, 1, "plan_available", true))
                } else {
                    return res.send(formatter.formatResponse({}, 0, "plan_not_found", false))
                }
            } else {
                if (availableRecipes && availableRecipes.length > 0) {
                    let recipes = availableRecipes.at(0)
                    let supplement = getSupplement(recipes.speciesId.name, weight, age, isNeutered, activityLevel);
                    supplementName = supplement.supplementName;
                    supplementDesc = supplement.supplementDesc;
                    let calories = calculateCalories(weight, activityLevel)
                    purchase = new RecipesPurchaseMaster({
                        userId: req.user._id, speciesId: speciesId, recipes: recipes._id, isPaid: isPurchase, breedId: breedId, calories: calories,
                        supplementName: supplementName, activityLevel: activityLevel, supplementDesc: supplementDesc, isNeutered: isNeutered, weight: weight, age: age
                    })
                    await purchase.save()
                    purchase = await RecipesPurchaseMaster.findOne({ _id: purchase._id }).populate('recipes', 'activityLevel title ingredients instructions').populate('speciesId', 'name').populate('breedId', 'name')
                    res.send(formatter.formatResponse(purchase, 1, "list_success", true))
                } else {
                    res.send(formatter.formatResponse({}, 0, "plan_not_found", false))
                }
            }
        }
    } catch (errorCode) {
        // console.log("=====errorCode=====" + errorCode.message);
        res.send(formatter.formatResponse({}, 0, errorCode, false))
    }

}

function getSupplement(name, weight, age, isSpayedOrNeutered, activityLevel) {
    let supplementName = "";
    let supplementDesc = "";
    if (name == 'Dog') {

        let omega3Amount = 0;
        // if (weight > 20) omega3Amount *= 1.5; // Increase for larger dogs
        omega3Amount = weight * 50;
        if (isSpayedOrNeutered) {
            omega3Amount *= 0.8; // Decrease if spayed/neutered
        }

        if (activityLevel == 'High' || age > 7) {
            let glucosamineAmount = weight * 25;
            // if (weight > 20) glucosamineAmount *= 1.5;
            supplementName = "Glucosamine"
            supplementDesc = glucosamineAmount + " mg to support joint health."
        }

        let vitaminBAmount = 0;
        // if (activityLevel == 'High') {
        vitaminBAmount = weight * 2;
        // }
        supplementName = "Glucosamine"
        supplementDesc = vitaminBAmount + " mg to support energy metabolism."
    }

    if (name == 'Cat') {
        let taurineAmount = 0;
        // if (weight > 5) taurineAmount *= 1.2; // Increase for larger cats
        taurineAmount = weight * 20;

        supplementName = "Taurine"
        supplementDesc = taurineAmount + " mg to support heart and eye health."

        // Omega-3 for skin and coat health
        let omega3Amount = 250;
        omega3Amount = weight * 50;
        // if (activityLevel == 'High') {
        //     omega3Amount *= 1.2; // Increase for active cats
        // }


        supplementName = "Omega-3 Fatty Acids"
        supplementDesc = omega3Amount + " mg to support skin, coat, and joint health."


        let vitaminBAmount = 250;
        vitaminBAmount = weight * 2;
        // if (activityLevel == 'High') {
        //     vitaminBAmount *= 1.2; // Increase for active cats
        // }

        supplementName = "Vitamin B Complex"
        supplementDesc = vitaminBAmount + " mg to support energy metabolism."
    }
    return { supplementName, supplementDesc };
}

function calculateCalories(weight, activityLevel) {
    // Calculate baseRER using the weight and exponentiation (**)
    let baseRER = 70 * Math.pow(weight, 0.75);
    let activityMultiplier;

    // Set the activityMultiplier based on the activity level
    switch (activityLevel) {
        case 'Low':
            activityMultiplier = 1.2;
            break;
        case 'Moderate':
            activityMultiplier = 1.6;
            break;
        case 'High':
            activityMultiplier = 2.0;
            break;
        default:
            activityMultiplier = 1.0;
    }
    let totalCalories = baseRER * activityMultiplier;
    return totalCalories.toFixed(2);
}


module.exports = userHomeController;