const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const BreedRecipesMaster = new Schema(
    {
        speciesId: { type: mongoose.Types.ObjectId, default: null, ref: "breed_category_master", index: 1 },
        activityLevel: { type: String,enum:['Moderate','High','Low'], required: [true, "Please enter activity level"] },
        title: { type: String, required: [true, "Please enter title"] },
        ingredients: { type: String, required: [true, "Please enter ingredients"] },
        instructions: { type: String, required: [true, "Please enter instructions"] },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: null },
        deleted_at: { type: Date, default: null },
    },
    {
        collection: "breed_recipes_master",
        name: 1,
        collation: { locale: "en", strength: 2 },
    }
);

BreedRecipesMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};


BreedRecipesMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;
    return obj;
};

module.exports = mongoose.model("breed_recipes_master", BreedRecipesMaster);
