const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const CartMaster = new Schema(
    {
        user: { type: mongoose.Types.ObjectId, default: null, ref: "user_master", index: 1 },
        product_arr: [
            {
                product_id: { type: mongoose.Types.ObjectId, ref: "product_master", index: 1 },
                quantity: { type: Number, default: 1 },
                note: { type: String, default: "" },
                isEdit: { type: Boolean, default: false }
            },
        ],
        subtotal: { type: Number, required: [true, "Please enter sub_total"] },
        shipping: { type: Number, default: 0 },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: Date.now },
    },
    {
        collection: "cart_master",
        // name: 1,
        // collation: { locale: "en", strength: 2 },
    }
);

CartMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

// const publicPath = APP_URL + "/public/";

CartMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;

    return obj;
};

module.exports = mongoose.model("cart_master", CartMaster);
