const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const CategoryMaster = new Schema(
    {
        category_id: { type: mongoose.Types.ObjectId, default: null, ref: "category_master", index: 1 },
        name: { type: String, required: [true, "Please enter Name"] },
        category_image: { type: String, default: null },
        cat_type: { type: String,  enum: ["doctor", "user"], require },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: null },
        deleted_at: { type: Date, default: null },
    },
    {
        collection: "category_master",
        name: 1,
        collation: { locale: "en", strength: 2 },
    }
);

CategoryMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

const publicPath = APP_URL + "/public/";

CategoryMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;
    if (obj.category_image && obj.category_image != "") {
        obj.category_image = publicPath + "category/" + obj.category_image;
    }
    return obj;
};

module.exports = mongoose.model("category_master", CategoryMaster);
