const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const ConsultationMaster = new Schema(
    {
        user: { type: mongoose.Types.ObjectId, required: [true, "Please enter User id"], ref: "user_master", index: 1 },
        doctor: { type: mongoose.Types.ObjectId, ref: "user_master", index: 1 },
        pet: { type: mongoose.Types.ObjectId, required: [true, "Please enter Pet id"], ref: "pet_master", index: 1 },
        issue: { type: String, required: [true, "Please enter Issue"] },
        chargeId: { type: String,default:null },
        schedule_time: { type: Date, required: [true, "Please enter time"] },
        slot: { type: String },
        fee: { type: Number, required: [true, "Please enter fee"] },
        appointment_type: { type: String, enum: ["Instant", "Scheduled"], default: "Instant" },
        description: { type: String, default: null },
        status: { type: String, enum: ["pending", "accepted", "rejected", "completed"], default: "pending" },
        priscribe: { type: mongoose.Types.ObjectId, default: null },
        image: [{ image: { type: String, default: null } }],
        payment: { type: String, default: null },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: Date.now },
    },
    {
        collection: "consultation_master",
        // name: 1,
        // collation: { locale: "en", strength: 2 },
    }
);

ConsultationMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

const publicPath = APP_URL + "/public/";

ConsultationMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    // delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;
    if (obj.image) {
        obj.image.forEach((element, key) => {
            obj.image[key].image = publicPath + "pet/" + element.image
        })
    }
    if (obj.pet) {
        obj.pet.pet_profile = publicPath + "pet/" + obj.pet.pet_profile
    }

    return obj;
};

module.exports = mongoose.model("consultation_master", ConsultationMaster);
