const mongoose = require('mongoose');
const Schema = mongoose.Schema;

const notificationMaster = new Schema({
    sender_id: { type: Schema.Types.ObjectId, ref: 'user_master', require: true, index: 1 },
    receiver_id: { type: Schema.Types.ObjectId, ref: 'user_master', require: true, index: 1 },
    pet_id: { type: Schema.Types.ObjectId, index: 1 },
    consultation_id: { type: Schema.Types.ObjectId, ref: 'consultation_master', index: 1 },
    msg: { type: String, require: [true, "Please enter message"] },
    msg_type: { type: String, enum: ["text", "image"], require: [true, "Please enter message type"] },
    appointment_type: { type: String },
    status: { type: String, default: "pending" },
    created_at: { type: Date, default: Date.now },
    updated_at: { type: Date, default: Date.now },
}, { collection: 'notification_master' });



notificationMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id, fields = "", sort = "") {
        return this.findById(id).populate("sender_id", "fname lname profile tag_id is_verified device_token user_type").populate("receiver_id", "fname lname profile tag_id is_verified device_token user_type").select(fields).sort(sort).exec();
    },
    /**
     * List data
     * @param {*} filter
     * @returns
     */
    async list(filter, fields = "", sort = "") {
        return this.find(filter).populate("sender_id", "fname lname profile tag_id is_verified device_token user_type").populate("receiver_id", "fname lname profile tag_id is_verified device_token user_type").select(fields).sort(sort).exec();
    }
}



notificationMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });
    const profilePath = global.APP_URL + "/public/profile/";

    obj.sender = obj.sender_id;
    if (obj.sender && obj.sender.profile && obj.sender.profile != "") {
        obj.sender.profile = profilePath + obj.sender.profile;
    }
    delete obj.sender_id;

    obj.receiver = obj.receiver_id;
    if (obj.receiver && obj.receiver.profile && obj.receiver.profile != "") {
        obj.receiver.profile = profilePath + obj.receiver.profile;
    }
    delete obj.receiver_id;

    // delete obj.created_at;
    delete obj.__v;
    return obj;
}

module.exports = mongoose.model('notification_master', notificationMaster);