const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const OrderMaster = new Schema(
    {
        user: { type: mongoose.Types.ObjectId, required: [true, "Please enter User id"], ref: "user_master", index: 1 },
        address: { type: mongoose.Types.ObjectId, required: [true, "Please enter Address id"], ref: "address_master", index: 1 },
        product_arr: [
            {
                product_id: { type: mongoose.Types.ObjectId, ref: "order_product_master", index: 1 },
                quantity: { type: Number, default: 1 },
                price: { type: Number, default: 0 }
            },
        ],
        subtotal: { type: Number, required: [true, "Please enter sub_total"] },
        shipping: { type: Number, default: 0 },
        order_status: { type: String, enum: ["Processed", "Dispached", "Delivered"], default: "Processed" },
        payment_method: { type: String, enum: ["cash_on_delivery", "online_payment"], default: "cash_on_delivery" },
        payment: { type: String, default: "pending", enum: ["pending", "success"] },
        total: { type: Number, required: [true, "Please enter total"] },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: Date.now },
    },
    {
        collection: "order_master",
        // name: 1,
        // collation: { locale: "en", strength: 2 },
    }
);

OrderMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

// const publicPath = APP_URL + "/public/";

OrderMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;

    return obj;
};

module.exports = mongoose.model("order_master", OrderMaster);
