const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const OrderProductMaster = new Schema(
    {
        order_id: { type: mongoose.Types.ObjectId, default: null, ref: "order_master", index: 1 },
        product_id: { type: mongoose.Types.ObjectId, default: null, index: 1 },
        name: { type: String, required: [true, "Please enter Name"] },
        product_image: { type: String },
        sku: { type: String, default: null },
        price: { type: Number, required: [true, "Please enter Price"] },
        description: { type: String, required: [true, "Please enter Description"] },
        productType: { type: String, enum: ["doctor", "user"], require },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: Date.now },
        deleted_at: { type: Date, default: null },
    },
    {
        collection: "order_product_master",
        name: 1,
        collation: { locale: "en", strength: 2 },
    }
);

OrderProductMaster.pre('find', function () {

    if (this.options.ignorePreHooks) {
        return;
    }
    // this.where({ deleted_at: null,is_hide: false ,is_suspended: false});
    this.where({ deleted_at: null });
});

OrderProductMaster.pre('findOne', function () {
    if (this.options.ignorePreHooks) {
        return;
    }
    this.where({ deleted_at: null });
});

OrderProductMaster.pre('findById', function () {

    if (this.options.ignorePreHooks) {
        return;
    }
    this.where({ deleted_at: null });
});

OrderProductMaster.pre('findByIdAndUpdate', function () {

    if (this.options.ignorePreHooks) {
        return;
    }
    this.where({ deleted_at: null });
});
OrderProductMaster.pre('findOneAndUpdate', function () {
    if (this.options.ignorePreHooks) {
        return;
    }
    this.where({ deleted_at: null, is_suspended: false });
});

OrderProductMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

const publicPath = APP_URL + "/public/";

OrderProductMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    // delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;
    if (obj.product_image && obj.product_image != "") {
        obj.product_image = publicPath + 'product/' + obj.product_image;
    }

    return obj;
};

module.exports = mongoose.model("order_product_master", OrderProductMaster);
