const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const PetMaster = new Schema(
    {
        user_id: { type: mongoose.Types.ObjectId, default: null, ref: "category_master", index: 1 },
        name: { type: String, required: [true, "Please enter Name"] },
        breed: { type: String, default: null },
        pet_type: { type: String, required: [true, "Please enter Type of pet"] },
        gender: { type: String, enum: ["Male", "Female"], default: "Male" },
        weight: { type: Number, default: 0 },
        dob: { type: Date, required: [true, "Please enter Date of birth"] },
        pet_profile: { type: String, default: null },
        pet_document: { type: String, default: null },
        spayed: { type: Boolean, default: false },
        image_arr: [{ image: { type: String, default: null } }],
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: Date.now },
        deleted_at: { type: Date, default: null },
    },
    { collection: "pet_master" }
);

PetMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

const publicPath = APP_URL + "/public/";

PetMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;
    if (obj.pet_profile && obj.pet_profile != "") {
        obj.pet_profile = publicPath + 'pet/' + obj.pet_profile;
    }
    if (obj.pet_document && obj.pet_document != "") {
        obj.pet_document = publicPath + 'pet/' + obj.pet_document;
    }
    // if (obj.image_arr && obj.image_arr.length > 0) {
    //     obj.image_arr = obj.image_arr.map((url) => {
    //         return { image: publicPath + 'pet/' + url.image, _id: url._id };
    //     });
    // }

    return obj;
};

module.exports = mongoose.model("pet_master", PetMaster);
