const mongoose = require("mongoose");
const Schema = mongoose.Schema;

const UserMaster = new Schema(
    {
        fname: { type: String, required: [true, "Please enter fname"] },
        lname: { type: String, required: [true, "Please enter lname"] },
        email: { type: String, required: [true, "Please enter email"] },
        phone: { type: Number, required: [true, "Please enter phone"] },
        ccode: { type: Number, required: [true, "Please enter ccode"] },
        user_type: { type: String, enum: ["pet_parent", "veterinarian"], defalult: "pet_parent" },
        is_hide: { type: Boolean, default: false },
        is_suspended: { type: Boolean, default: false },
        pet_type: [{ type: String }],
        profile: { type: String, default: null },
        specialization: { type: String, default: null },
        price: { type: Number, default: null },
        bio: { type: String, default: null },
        document: { type: String, default: null },
        device_type: { type: String, enum: ["iOS", "android"], default: "android" },
        device_token: { type: String, default: null },
        wallet: { type: Number, default: 0 },
        AccountId: { type: String, default: null },
        created_at: { type: Date, default: Date.now },
        updated_at: { type: Date, default: null },
        deleted_at: { type: Date, default: null },
        reason: { type: String, default: null },
        is_on_chat: { type: String, default: null },
        address: { type: mongoose.Schema.Types.ObjectId, default: null, index: 1 }
    },
    {
        collection: "user_master",
        email: 1,
        collation: { locale: "en", strength: 2 },
    }
);

UserMaster.pre('find', function () {
    if (this.getQuery().customCondition) {
        return;
    }
    if (this.options.ignorePreHooks) {
        this.where({ deleted_at: null });
        return;
    }
    // this.where({ deleted_at: null,is_hide: false ,is_suspended: false});
    this.where({ deleted_at: null, is_suspended: false });
});

UserMaster.pre('findOne', function () {
    if (this.getQuery().customCondition) {
        return;
    }
    if (this.options.ignorePreHooks) {
        this.where({ deleted_at: null });
        return;
    }
    this.where({ deleted_at: null, is_suspended: false });
});

UserMaster.pre('findById', function () {
    if (this.getQuery().customCondition) {
        return;
    }
    if (this.options.ignorePreHooks) {
        this.where({ deleted_at: null });
        return;
    }
    this.where({ deleted_at: null, is_suspended: false });
});

UserMaster.pre('findByIdAndUpdate', function () {
    if (this.getQuery().customCondition) {
        return;
    }
    if (this.options.ignorePreHooks) {
        this.where({ deleted_at: null });
        return;
    }
    this.where({ deleted_at: null, is_suspended: false });
});
UserMaster.pre('findOneAndUpdate', function () {
    if (this.getQuery().customCondition) {
        return;
    }
    if (this.options.ignorePreHooks) {
        this.where({ deleted_at: null });
        return;
    }
    this.where({ deleted_at: null, is_suspended: false });
});


UserMaster.statics = {
    /**
     * Single data
     * @param {*} filter
     * @returns
     */
    async get(id) {
        return this.findById(id).exec();
    },

    /**
     * list data
     * @param {*} filter
     * @returns
     */
    async list(filter = {}) {
        return this.find(filter).exec();
    },
};

const publicPath = APP_URL + "/public/";

UserMaster.methods.toJSON = function () {
    var obj = this.toObject();
    let objKeys = Object.keys(obj);
    objKeys.forEach(function (key) {
        if (obj[key] == null) {
            obj[key] = "";
        }
    });

    delete obj.created_at;
    delete obj.updated_at;
    delete obj.deleted_at;
    delete obj.__v;
    if (obj.profile && obj.profile != "") {
        obj.profile = publicPath + "profile/" + obj.profile;
    }
    if (obj.document && obj.document != "") {
        obj.document = publicPath + "profile/" + obj.document;
    }
    return obj;
};

module.exports = mongoose.model("user_master", UserMaster);
