const config = require('../config/common.config');
const ResponseFormatter = require('../utils/response-formatter');
const formatter = new ResponseFormatter();
const jwt = require('jsonwebtoken');
const userController = require('../controllers/user.controller');
const VersionMaster = require('../models/VersionMaster');
const adminMaster = require('../models/AdminMaster');

var routeMiddlewares = {
    validateRequest: function (requestSchema) {
        return (req, res, next) => {
            req.user_Ip = req.socket.remoteAddress;
            // console.log(req.body);
            const validations = ['headers', 'params', 'query', 'body']
                .map(key => {
                    const schema = requestSchema[key];
                    const value = req[key];
                    if (schema) {
                        const { error } = schema.validate(value);
                        if (error) {
                            const { details } = error;
                            const message = details.map(i => i.message).join(',')

                            var finalRes = formatter.formatResponse({}, 0, message, false);
                            return res.status(422).send(finalRes);
                        }
                        else {
                            next();
                        }
                    }
                });

        };
    },
    authorize: async function (req, res, next) {
        // TODO: check id user is active
        try {
            req.user_Ip = req.socket.remoteAddress;
            let token;
            let errorCode = "unknown_error";
            if (req.headers.authorization) {
                if (typeof req.headers.authorization !== 'string' || req.headers.authorization.indexOf('Bearer ') === -1) {
                    errorCode = "incorrect_token";
                } else {
                    token = req.headers.authorization.split(' ')[1];
                }
            } else if (req.headers && req.headers.token) {
                token = req.headers.token;
            } else {
                errorCode = "incorrect_token";
            }
            // console.log('token', token);

            if (!token && errorCode) {

                var finalRes = formatter.formatResponse({}, 3, config.messages[errorCode], false);
                return res.send(finalRes);
            }

            // console.log("========token=====",token);
            

            jwt.verify(token, config.jwt.secret, async (err, decoded) => {

                // console.log("========decoded=====",decoded);

                if (err || !decoded || !decoded._id) {
                    errorCode = "incorrect_token";
                    var finalRes = formatter.formatResponse({}, 3, config.messages[errorCode], false);
                    return res.status(401).send(finalRes);
                }
                // console.log('decoded', decoded);
                // TODO : enable this code if user context is required in auth protected APIs
                const user = await userController.getUserDetail(decoded._id);
                // console.log("=====user======",user);
                
                if (user == null) {
                    console.error("authorize failure - ");
                    var finalRes = formatter.formatResponse({}, 11, config.messages["your_account_deleted"], false);
                    return res.send(finalRes);
                }

                if (user.user_type != null && user.user_type == "veterinarian" && user.is_suspended == true) {
                    console.error("authorize failure - ");
                    var finalRes = formatter.formatResponse({}, 10, config.messages["doctor_is_suspend"], false);
                    return res.send(finalRes);
                }

                if (user.user_type != null && user.user_type == "pet_parent" && user.is_suspended == true) {
                    console.error("authorize failure - ");
                    var finalRes = formatter.formatResponse({}, 10, config.messages["user_is_suspend"], false);
                    return res.send(finalRes);
                }

                req.user = user;
                return next();
            });
        } catch (errorCode) {
            console.error("authorize failure - ", errorCode);
            var finalRes = formatter.formatResponse({}, 3, config.messages[errorCode], false);
            return res.send(finalRes);
        }
    },
    authAdmin: async function (req, res, next) {
        // TODO: check id user is active
        try {
            req.user_Ip = req.socket.remoteAddress;
            let token;
            let errorCode = "unknown_error";
            if (req.headers.authorization) {
                if (typeof req.headers.authorization !== 'string' || req.headers.authorization.indexOf('Bearer ') === -1) {
                    errorCode = "incorrect_token";
                } else {
                    token = req.headers.authorization.split(' ')[1];
                }
            } else if (req.headers && req.headers.token) {
                token = req.headers.token;
            } else {
                errorCode = "incorrect_token";
            }
            console.log('token==========', token);

            if (!token && errorCode) {

                var finalRes = formatter.formatResponse({}, 3, config.messages[errorCode], false);
                return res.send(finalRes);
            }

            jwt.verify(token, config.jwt.secret, async (err, decoded) => {
                console.log('decoded._id==========', decoded);
                if (err || !decoded || !decoded._id) {
                    errorCode = "incorrect_token";
                    var finalRes = formatter.formatResponse({}, 3, config.messages[errorCode], false);
                    return res.status(401).send(finalRes);
                }

                const admin = await adminMaster.findOne({ _id: decoded._id });
                if (admin == null) {
                    console.error("authorize failure - ");
                    var finalRes = formatter.formatResponse({}, 3, config.messages["incorrect_token"], false);
                    return res.send(finalRes);
                }
                req.user = admin;
                return next();
            });
        } catch (errorCode) {
            console.error("authorize failure - ", errorCode);
            var finalRes = formatter.formatResponse({}, 3, config.messages[errorCode], false);
            return res.send(finalRes);
        }
    },
    checkVersion: async function (req, res, next) {
        let device_version = req.headers.versioncode;
        let device_type = req.headers.devicetype;
        let current_version = await VersionMaster.findOne({ device_type: device_type }).exec();
        if (!device_version || !device_type) {
            var finalRes = formatter.formatResponse({}, 0, config.messages["require_headers"], false);
            return res.send(finalRes);
        }
        if (device_version && current_version.version_code > device_version.toString()) {
            var finalRes = formatter.formatResponse({}, 26, config.messages["incorrect_mode"], false);
            return res.send(finalRes);
        } else {
            return next();
        }
    }
};

module.exports = routeMiddlewares;