const veterinarianCtrl = require("../controllers/veterinarian.controller");
const config = require("../config/common.config");
const express = require("express");
const router = express.Router();
const veterinarianValidation = require("../validations/veterinarianValidations");
const routeMiddlewares = require("../routes/routeMiddlewares");
const multer = require("multer");
const fs = require("fs");
const publicPath = basedir + "/public/";

const multerStorage = multer.diskStorage({
    destination: (req, file, cb) => {
        if (file.fieldname == "profile" || file.fieldname == "document") {
            if (!fs.existsSync("public/profile")) {
                fs.mkdirSync("public/profile");
            }
            cb(null, "public/profile");
        }
        if (file.fieldname == "front_image" || file.fieldname == "back_image") {
            if (!fs.existsSync("public/document")) {
                fs.mkdirSync("public/document");
            }
            cb(null, "public/document");
        }
    },
    filename: (req, file, cb) => {
        const ext = file.mimetype.split("/")[1];
        const unique = Math.floor(Math.random() * (9999 - 1000 + 1)) + 1000
        cb(null, `img_${Date.now()}${unique}.${ext}`);
    },
});

const upload = multer({ storage: multerStorage });


// router.post(
//     "/refreshToken",
//     upload.none(),
//     routeMiddlewares.validateRequest(veterinarianValidation.userIdSchema),
//     veterinarianCtrl.refreshToken
// );

router.post("/sendOtp",
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.sendOtpSchema),
    veterinarianCtrl.sendOtp,
);

router.post("/signUp",
    upload.fields([{ name: 'profile' }, { name: 'document' }]),
    routeMiddlewares.validateRequest(veterinarianValidation.signupSchema),
    veterinarianCtrl.signUp,
);

router.post("/updateVeterinarian",
    routeMiddlewares.authorize,
    upload.fields([{ name: 'profile' }, { name: 'document' }]),
    routeMiddlewares.validateRequest(veterinarianValidation.updateVeterinarianSchema),
    veterinarianCtrl.updateVeterinarian,
);

router.post(
    "/verifyOTP",
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.verifyOTPSchema),
    veterinarianCtrl.verifyOTP
);

router.get(
    "/petTypeList",
    upload.none(),
    veterinarianCtrl.petTypeList
);

router.post(
    "/productList",
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.productListSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.productList
);

router.post(
    "/productSearch",
    upload.none(),
    // routeMiddlewares.validateRequest(veterinarianValidation.productSearchSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.productSearch
);

router.get(
    '/requestedAppointmentList',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.requestedAppointmentList,
)
router.get(
    '/upcomingAppointmentList',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.upcomingAppointmentList,
)

router.post(
    '/appointmentStatus',
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.appointmentStatusSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.appointmentStatus,
)

router.post(
    '/createPricribe',
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.createPricribeSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.createPricribe,
)
router.get(
    '/consultationHistory',
    upload.none(),
    // routeMiddlewares.validateRequest(veterinarianValidation.consultationHistorySchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.consultationHistory,
)
router.get(
    '/notificationList',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.notificationList,
)
router.post(
    '/acceptNotification',
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.acceptNotificationSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.acceptNotification,
)
router.post(
    '/walletAmount',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.walletAmount,
)
router.post(
    '/addBankDetail',
    upload.fields([{ name: 'front_image' }, { name: 'back_image' }]),
    routeMiddlewares.validateRequest(veterinarianValidation.addBankDetailSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.addBankDetail,
)
router.post(
    '/bankDetail',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.bankDetail,
)
router.post(
    '/updateBankDetail',
    upload.fields([{ name: 'front_image' }, { name: 'back_image' }]),
    routeMiddlewares.validateRequest(veterinarianValidation.updateBankDetailSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.updateBankDetail,
)
router.post(
    '/withdrawAmount',
    upload.none(),
    routeMiddlewares.validateRequest(veterinarianValidation.withdrawAmountSchema),
    routeMiddlewares.authorize,
    veterinarianCtrl.withdrawAmount,
)
router.post(
    '/payoutHistory',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.payoutHistory,
)
router.post(
    '/cronRefund',
    upload.none(),
    routeMiddlewares.authorize,
    veterinarianCtrl.cronRefund,
)
module.exports = router