const Joi = require('joi');
const emailSchema = Joi.string();
const stringSchema = Joi.string();

Joi.objectId = require('joi-objectid')(Joi)

module.exports = {
    login: {
        body: Joi.object().keys({
            email: emailSchema.email({ tlds: { allow: false } }).required(),
            password: stringSchema.required()
        })
    },
    forgotPasswordSchema: {
        body: Joi.object().keys({
            email: emailSchema.email({ tlds: { allow: false } }).required()
        })
    },
    verifyOTPSchema: {
        body: Joi.object().keys({
            email: emailSchema.email({ tlds: { allow: false } }).required(),
            otp: Joi.string().length(4).required()
        })
    },
    resetPasswordSchema: {
        body: Joi.object().keys({
            email: emailSchema.email({ tlds: { allow: false } }).required(),
            new_password: stringSchema.required(),                            // .min(4)
        })
    },
    userSchema: {
        body: Joi.object().keys({
            page: Joi.number(),
            limit: Joi.number(),
            user_type: stringSchema.allow("pet_parent", "veterinarian", ""),
            search: Joi.string().allow("")
        })
    },
    categorySchema: {
        body: Joi.object().keys({
            category_id: Joi.objectId().allow(""),
            catType: stringSchema.valid("doctor", "user").required(),
            name: stringSchema.required(),
            category_image: Joi.any()
        })
    },
    categoryListSchema: {
        body: Joi.object().keys({
            category_id: Joi.objectId().allow(""),
            catType: stringSchema.valid("doctor", "user").required(),
            page: Joi.number(),
            limit: Joi.number(),
            search: Joi.string().allow("")
        })
    },
    categoryUpdateSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().allow(""),
            category_id: Joi.objectId().allow(""),
            name: stringSchema.required(),
            category_image: Joi.any()
        })
    },
    categoryDeleteSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required(),
        })
    },
    productSchema: {
        body: Joi.object().keys({
            category_id: Joi.objectId().required(),
            sub_category_id: Joi.objectId().required(),
            name: stringSchema.required(),
            product_image: Joi.any(),
            price: stringSchema.required(),
            description: stringSchema.required(),
            productType: stringSchema.valid("doctor", "user").required(),
            sku: stringSchema.required(),
            quantity: Joi.number().required()
        })
    },
    productUpdateSchema: {
        body: Joi.object().keys({
            product_id: Joi.objectId().required(),
            category_id: Joi.objectId().required(),
            sub_category_id: Joi.objectId().required(),
            name: stringSchema,
            product_image: Joi.any(),
            price: stringSchema,
            description: stringSchema,
            sku: stringSchema.required(),
            quantity: Joi.number()
        })
    },
    productDeleteSchema: {
        body: Joi.object().keys({
            product_id: Joi.objectId().required()
        })
    },
    productListSchema: {
        body: Joi.object().keys({
            sub_category_id: Joi.objectId().allow(""),
            page: Joi.number(),
            limit: Joi.number(),
            productType: stringSchema.valid("doctor", "user").required(),
            search: stringSchema.allow("")
        })
    },
    createPetTypeSchema: {
        body: Joi.object().keys({
            pet_image: Joi.any(),
            pet_name: stringSchema.required()
        })
    },
    createPetBreedSpeciesSchema: {
        body: Joi.object().keys({
            name: stringSchema.required(),
            type: stringSchema.valid("species", "breed").required(),
            pet_type_id: Joi.objectId().required()
        })
    },

    updatePetTypeSchema: {
        body: Joi.object().keys({
            pet_id: Joi.objectId().required(),
            pet_image: Joi.any(),
            pet_name: stringSchema.allow('')
        })
    },
    deletePetTypeSchema: {
        body: Joi.object().keys({
            pet_id: Joi.objectId().required(),
        })
    },
    orderListSchema: {
        body: Joi.object().keys({
            page: Joi.number().allow(""),
            limit: Joi.number().allow(""),
            search: stringSchema.allow("")
        })
    },
    prececriptionListSchema: {
        body: Joi.object().keys({
            doctor_id: Joi.string().required(),
            search: Joi.string().allow(""),
            page: Joi.number().allow(""),
            limit: Joi.number().allow("")
        })
    },
    editDoctorInfoSchema: {
        body: Joi.object().keys({
            doctor_id: Joi.string().required(),
            fname: Joi.string().allow(""),
            lname: Joi.string().allow(""),
            specialization: Joi.string().allow(""),
            price: Joi.string().allow(""),
            bio: Joi.string().allow(""),
            profile: Joi.any()
        })
    },
    editUserInfoSchema: {
        body: Joi.object().keys({
            userId: Joi.string().required(),
            fname: Joi.string().allow(""),
            lname: Joi.string().allow(""),
            profile: Joi.any(),
            pet_type: Joi.string().allow("")
        })
    },
    deleteDoctorSchema: {
        body: Joi.object().keys({
            doctor_id: Joi.string().required(),
            reason: Joi.string().required()
        })
    },
    hideUnHideDoctor: {
        body: Joi.object().keys({
            doctor_id: Joi.string().required(),
            isHide: Joi.boolean()
        })
    },
    suspendUnsuspendDoctor: {
        body: Joi.object().keys({
            doctor_id: Joi.string().required(),
            isSuspend: Joi.boolean()
        })
    },
    petDetailSchema: {
        body: Joi.object().keys({
            userId: Joi.string().required(),
        })
    },
    updateOrderSchema: {
        body: Joi.object().keys({
            orderId: Joi.objectId().required(),
            orderStatus: Joi.string().valid("Processed", "Dispached", "Delivered").required()
        })
    },
    orderDetailSchema: {
        body: Joi.object().keys({
            orderId: Joi.objectId().required()
        })
    },
    getpayoutListSchema: {
        body: Joi.object().keys({
            page: Joi.number().optional().allow(""),
            limit: Joi.number().optional().allow(""),
            search: stringSchema.optional().allow("")
        })
    },
    payoutStatusUpdateSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required(),
            status: stringSchema.required().valid("accept", "reject")
        })
    }
}