const Joi = require('joi');
const emailSchema = Joi.string();
const stringSchema = Joi.string();

Joi.objectId = require('joi-objectid')(Joi)

module.exports = {
    signupSchema: {
        body: Joi.object().keys({
            fname: stringSchema.required(),
            lname: stringSchema.required(),
            email: emailSchema.required(),
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            ccode: Joi.number().required(),
            otp: Joi.number().required(),
            pet_type: Joi.array().required(),
            profile: Joi.any(),
            device_type: stringSchema,
            device_token: stringSchema,
        })
    },
    updateProfileSchema: {
        body: Joi.object().keys({
            user_id: Joi.objectId(),
            fname: stringSchema,
            lname: stringSchema,
            email: emailSchema,
            pet_type: Joi.array(),
            profile: Joi.any()
        })
    },
    verifyOTPSchema: {
        body: Joi.object().keys({
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            ccode: Joi.number().required(),
            otp: Joi.number().required(),
            device_token: stringSchema.required(),
            device_type: stringSchema.required()
        })
    },
    logoutSchema: {
        body: Joi.object().keys({
            user_id: Joi.objectId().required()
        })
    },
    deleteAccountSchema: {
        body: Joi.object().keys({
            reason: stringSchema.required()
        })
    },
    userIdSchema: {
        body: Joi.object().keys({
            user_id: Joi.objectId().required()
        })
    },
    changePassSchema: {
        body: Joi.object().keys({
            user_id: Joi.objectId().required(),
            old_password: stringSchema.required(),
            new_password: stringSchema.required(),
        })
    },
    sendOtpSchema: {
        body: Joi.object().keys({
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            ccode: Joi.number().required(),
            email: emailSchema.allow(""),
            is_signup: stringSchema.required(),
        })
    },
    updatePassSchema: {
        body: Joi.object().keys({
            new_password: stringSchema.required(),
            user_id: Joi.objectId().required(),
        })
    },
    contactUsSchema: {
        body: Joi.object().keys({
            name: stringSchema.required(),
            email: emailSchema.required(),
            subject: stringSchema.required(),
            message: stringSchema.required(),
        })
    },


    addressSchema: {
        body: Joi.object().keys({
            fname: stringSchema.required(),
            lname: stringSchema.required(),
            company_name: stringSchema.optional(),
            country: stringSchema.required(),
            line_1: stringSchema.required(),
            line_2: stringSchema.optional(),
            city: stringSchema.required(),
            state: stringSchema.optional(),
            zipcode: stringSchema.optional(),
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            email: emailSchema.required(),
            notes: stringSchema.optional(),
            is_default: stringSchema,
        })
    },
    updateAddressSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required(),
            fname: stringSchema.optional(),
            lname: stringSchema.optional(),
            company_name: stringSchema.optional(),
            country: stringSchema.optional(),
            line_1: stringSchema.optional(),
            line_2: stringSchema.optional(),
            city: stringSchema.optional(),
            state: stringSchema.optional(),
            zipcode: stringSchema.optional(),
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).optional(),
            email: emailSchema.optional(),
            notes: stringSchema.optional(),
            is_default: stringSchema.optional(),
        })
    },
    deleteAddressSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required()
        })
    },
    productDeatilSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required(),
        })
    },
    catTypeSchema: {
        body: Joi.object().keys({
            catType: stringSchema.required()
        })
    },
    productListSchema: {
        body: Joi.object().keys({
            sub_category_id: Joi.array(),
            page: stringSchema.allow(''),
            search: stringSchema.allow(''),
            limit: stringSchema.allow(''),
            productType: stringSchema.valid("doctor", "user").required(),
        })
    },
    addToCartSchema: {
        body: Joi.object().keys({
            product_id: Joi.string().required(),
            quantity: stringSchema.required(),
            note: stringSchema,
            isEdit: Joi.boolean().default(false).optional(),
            precribe_id: Joi.objectId().optional().allow('')
        })
    },
    placeOrderSchema: {
        body: Joi.object().keys({
            payment_method: stringSchema.required(),
            address_id: Joi.objectId().required()
        })
    },
    updateCartSchema: {
        body: Joi.object().keys({
            product_id: Joi.objectId().required(),
            quantity: Joi.number().required()
        })
    },
    petProfileSchema: {
        body: Joi.object().keys({
            name: stringSchema.required(),
            breed: stringSchema.allow(''),
            pet_type: stringSchema.required(),
            gender: stringSchema.required(),
            weight: stringSchema,
            dob: stringSchema.required(),
            spayed: stringSchema,
            pet_profile: Joi.any(),
            pet_document: Joi.any(),
            image_arr: Joi.any(),
            delete_image_id: Joi.any()
        })
    },
    updatePetProfileSchema: {
        body: Joi.object().keys({
            pet_id: Joi.objectId().required(),
            // delete_image_id: Joi.array().allow(''),
            delete_image_id: stringSchema.allow(''),
            name: stringSchema,
            breed: stringSchema,
            pet_type: stringSchema,
            gender: stringSchema,
            weight: stringSchema,
            dob: stringSchema,
            spayed: stringSchema,
            pet_profile: Joi.any(),
            pet_document: Joi.any(),
            image_arr: Joi.any(),
        })
    },
    deletePetProfileSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required(),
        })
    },
    consultationSchema: {
        body: Joi.object().keys({
            doctor: Joi.objectId().allow(''),
            pet: Joi.objectId().required(),
            issue: stringSchema.required(),
            schedule_time: stringSchema,
            slot: stringSchema.allow(''),
            // fee: stringSchema.required(),
            appointment_type: stringSchema.required(),
            description: stringSchema.allow(''),
            image: Joi.any()
        })
    },
    doctorSchema: {
        body: Joi.object().keys({
            page: stringSchema.allow(""),
            limit: stringSchema.allow(""),
            search: stringSchema.allow(""),
        })
    },
    // orderPlacePrescribeSchema: {
    //     body: Joi.object().keys({
    //         address: Joi.objectId().required(),
    //         product_arr: Joi.array().items(Joi.object({
    //             product: Joi.objectId().required(),
    //             quantity: Joi.number().default(1),
    //             price: Joi.number().default(0),
    //         })),
    //         payment_method: stringSchema.required()
    //     })
    // },
    chatAiSchema: {
        body: Joi.object().keys({
            message: stringSchema.required(),
            phm_id: stringSchema.allow('')
        })
    },
    petNutritionSchema: {
        body: Joi.object().keys({
            gender: stringSchema.required(),
            breed: stringSchema.required(),
            species: stringSchema.required(),
            dateOfBirthMillis: Joi.number().required(),
            neutered: Joi.boolean().required(),
            weight: Joi.number().required()
        })
    },
    consultationDetailSchema: {
        body: Joi.object().keys({
            consultation_id: Joi.array().required()
        })
    },
    consultationSlotSchema: {
        body: Joi.object().keys({
            doctor_id: Joi.objectId().required(),
            date: stringSchema.required()
        })
    },
    consultationCloseSchema: {
        body: Joi.object().keys({
            consultation_id: Joi.array().required(),
        })
    },
    refundPaymentSchema: {
        body: Joi.object().keys({
            consultation_id: Joi.objectId().required(),
        })
    },
    recipesSchema: {
        body: Joi.object().keys({
            speciesId: Joi.objectId(),
            activityLevel: stringSchema.required()
        })
    },
    purchaseRecipeSchema: {
        body: Joi.object().keys({
            isPurchase: Joi.boolean().default(false),
            isNeutered: Joi.boolean().default(false),
            speciesId: Joi.objectId().required(),
            breedId: Joi.objectId().required(),
            activityLevel: stringSchema.required().valid("High", "Moderate", "Low"),
            weight: Joi.number().optional(),
            age: Joi.number().optional(),
            isCheck: Joi.boolean().optional()
        })
    }
};