const Joi = require('joi');
const emailSchema = Joi.string();
const stringSchema = Joi.string();

Joi.objectId = require('joi-objectid')(Joi)

module.exports = {
    signupSchema: {
        body: Joi.object().keys({
            fname: stringSchema.required(),
            lname: stringSchema.required(),
            email: emailSchema.required(),
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            ccode: Joi.number().required(),
            otp: Joi.number().required(),
            pet_type: Joi.array().required(),
            profile: Joi.any(),
            specialization: stringSchema.required(),
            price: stringSchema.required(),
            bio: stringSchema.required(),
            document: Joi.any(),
            device_type: stringSchema,
            device_token: stringSchema,
        })
    },
    updateVeterinarianSchema: {
        body: Joi.object().keys({
            user_id: Joi.objectId().required(),
            fname: stringSchema,
            lname: stringSchema,
            profile: Joi.any(),
            document: Joi.any(),
            specialization: stringSchema,
            price: stringSchema,
            bio: stringSchema,
            pet_type: Joi.array()
        })
    },
    userIdSchema: {
        body: Joi.object().keys({
            user_id: Joi.objectId().required()
        })
    },
    productListSchema: {
        body: Joi.object().keys({
            productType: stringSchema.valid("doctor", "user").required(),
        })
    },
    sendOtpSchema: {
        body: Joi.object().keys({
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            ccode: Joi.number().required(),
            email: emailSchema.allow(""),
            is_signup: Joi.boolean().required(),
        })
    },
    verifyOTPSchema: {
        body: Joi.object().keys({
            phone: stringSchema.min(5).max(15).pattern(/^[0-9]+$/).required(),
            ccode: Joi.number().required(),
            otp: Joi.number().required(),
            device_token: stringSchema.required(),
            device_type: stringSchema.required()
        })
    },
    appointmentStatusSchema: {
        body: Joi.object().keys({
            _id: Joi.objectId().required(),
            sender_id: Joi.objectId().required(),
            status: stringSchema.required()
        })
    },
    createPricribeSchema: {
        body: Joi.object().keys({
            user: Joi.objectId().required(),
            consultation: Joi.objectId().required(),
            product_arr: Joi.array().items(Joi.object({
                product: Joi.objectId().required(),
                quantity: Joi.number().default(1),
                price: Joi.number().default(0),
                note: Joi.object().keys({
                    medication_name: stringSchema.allow(''),
                    notes: stringSchema.allow('')
                }),
            }))
        })
    },
    acceptNotificationSchema: {
        body: Joi.object().keys({
            notifi_id: Joi.objectId().required(),
            status: stringSchema.required()
        })
    },
    addBankDetailSchema: {
        body: Joi.object().keys({
            bank_name: stringSchema.required(),
            email: stringSchema.required(),
            country_code: stringSchema.required(),
            phone: stringSchema.required(),
            dob: stringSchema.required(),
            identify_number: stringSchema.required(),
            city: stringSchema.required(),
            country: stringSchema.required(),
            address: stringSchema.required(),
            postal_code: stringSchema.required(),
            state: stringSchema.required(),
            external_account_country: stringSchema.required(),
            external_account_currency: stringSchema.required(),
            bank_account_number: stringSchema.required(),
            routing_number: stringSchema.required(),
            front_image: Joi.any(),
            back_image: Joi.any(),
        })
    },
    updateBankDetailSchema: {
        body: Joi.object().keys({
            bank_name: stringSchema.allow(''),
            email: stringSchema.allow(''),
            dob: stringSchema.allow(''),
            identify_number: stringSchema.allow(''),
            city: stringSchema.allow(''),
            country: stringSchema.allow(''),
            address: stringSchema.allow(''),
            postal_code: stringSchema.allow(''),
            state: stringSchema.allow(''),
            external_account_country: stringSchema.allow(''),
            external_account_currency: stringSchema.allow(''),
            bank_account_number: stringSchema.allow(''),
            routing_number: stringSchema.allow(''),
            front_image: Joi.any(),
            back_image: Joi.any(),
        })
    },
    withdrawAmountSchema: {
        body: Joi.object().keys({
            amount: stringSchema.required(),
        })
    },
}